@@ -22,7 +22,7 @@
 **	27 Jan 95 Ensured that proxy service will be overridden for files
 **		  on the local host (because HTLoadFile() doesn't try ftp
 **		  for those) and will substitute ftp for remote files. FM
-**	28 Jan 95 Tweeked PDM's proxy override mods to handle port info
+**	28 Jan 95 Tweaked PDM's proxy override mods to handle port info
 **		  for news and wais URL's. FM
 **
 **  Bugs
@@ -64,14 +64,13 @@
 #include <LYUtils.h>
 #include <LYLeaks.h>
 
-extern HTCJKlang HTCJK;
-
 /*
 **  These flags may be set to modify the operation of this module
 */
 PUBLIC char * HTClientHost = NULL; /* Name of remote login host if any */
 PUBLIC FILE * HTlogfile = NULL;    /* File to which to output one-liners */
 PUBLIC BOOL HTSecure = NO;	   /* Disable access for telnet users? */
+PUBLIC BOOL HTPermitRedir = NO;	   /* Always allow redirection in getfile()? */
 
 PUBLIC BOOL using_proxy = NO; /* are we using a proxy gateway? */
 
@@ -270,13 +269,8 @@ PUBLIC BOOL override_proxy ARGS1(
 
     if ((acc_method = HTParse(addr, "", PARSE_ACCESS))) {
 	if (!strcmp("file", acc_method) &&
-	    (!strcmp(Host, "localhost") ||
-#ifdef VMS
-	     !strcasecomp(Host, HTHostName())
-#else
-	     !strcmp(Host, HTHostName())
-#endif /* VMS */
-	)) {
+	    (LYSameHostname(Host, "localhost") ||
+	     LYSameHostname(Host, HTHostName()))) {
 	    FREE(host);
 	    FREE(acc_method);
 	    return YES;
@@ -315,6 +309,9 @@ PUBLIC BOOL override_proxy ARGS1(
 #ifndef DISABLE_FINGER
 	    else if (!strcmp(acc_method, "finger"))	port = 79;
 #endif
+	    else if (!strcmp(acc_method, "telnet"))	port = 23;
+	    else if (!strcmp(acc_method, "tn3270"))	port = 23;
+	    else if (!strcmp(acc_method, "rlogin"))	port = 513;
 	    FREE(acc_method);
 	}
     }
@@ -351,6 +348,15 @@ PUBLIC BOOL override_proxy ARGS1(
 	    FREE(host);
 	    return YES;
 	}
+#ifdef CJK_EX	/* ASATAKU PROXY HACK */
+	if ((!templ_port || templ_port == port)	 &&
+	    (t_len > 0	&&  t_len <= h_len  &&
+	     isdigit(UCH(*no_proxy)) && !strncmp(host, no_proxy, t_len))) {
+	    FREE(host);
+	    return YES;
+	}
+#endif	/* ASATAKU PROXY HACK */
+
 	if (*end)
 	    no_proxy = (end + 1);
 	else
@@ -376,26 +382,45 @@ PRIVATE int get_physical ARGS2(
 	CONST char *,		addr,
 	HTParentAnchor *,	anchor)
 {
+    int result;
     char * acc_method = NULL;	/* Name of access method */
     char * physical = NULL;
     char * Server_addr = NULL;
+    BOOL override_flag = NO;
+
+    CTRACE((tfp, "get_physical %s\n", addr));
+
+    /*
+    **	Make sure the using_proxy variable is FALSE.
+    */
+    using_proxy = NO;
 
 #ifndef NO_RULES
-    physical = HTTranslate(addr);
-    if (!physical) {
+    if ((physical = HTTranslate(addr)) == 0) {
+	if (redirecting_url) {
+	    return HT_REDIRECTING;
+	}
 	return HT_FORBIDDEN;
     }
     if (anchor->isISMAPScript == TRUE) {
 	StrAllocCat(physical, "?0,0");
-	CTRACE(tfp, "HTAccess: Appending '?0,0' coordinate pair.\n");
+	CTRACE((tfp, "HTAccess: Appending '?0,0' coordinate pair.\n"));
+    }
+    if (!strncmp(physical, "Proxied=", 8)) {
+	HTAnchor_setPhysical(anchor, physical + 8);
+	using_proxy = YES;
+    } else if (!strncmp(physical, "NoProxy=", 8)) {
+	HTAnchor_setPhysical(anchor, physical + 8);
+	override_flag = YES;
+    } else {
+	HTAnchor_setPhysical(anchor, physical);
     }
-    HTAnchor_setPhysical(anchor, physical);
     FREE(physical);			/* free our copy */
 #else
     if (anchor->isISMAPScript == TRUE) {
 	StrAllocCopy(physical, addr);
 	StrAllocCat(physical, "?0,0");
-	CTRACE(tfp, "HTAccess: Appending '?0,0' coordinate pair.\n");
+	CTRACE((tfp, "HTAccess: Appending '?0,0' coordinate pair.\n"));
 	HTAnchor_setPhysical(anchor, physical);
 	FREE(physical);			/* free our copy */
     } else {
@@ -403,72 +428,69 @@ PRIVATE int get_physical ARGS2(
     }
 #endif /* NO_RULES */
 
-    acc_method =  HTParse(HTAnchor_physical(anchor),
-		"file:", PARSE_ACCESS);
+    acc_method = HTParse(HTAnchor_physical(anchor), STR_FILE_URL, PARSE_ACCESS);
 
     /*
     **	Check whether gateway access has been set up for this.
     **
     **	This function can be replaced by the rule system above.
+    **
+    **  If the rule system has already determined that we should
+    **  use a proxy, or that we shouldn't, ignore proxy-related
+    **  settings, don't use no_proxy either.
     */
 #define USE_GATEWAYS
 #ifdef USE_GATEWAYS
-    /*
-    **	Make sure the using_proxy variable is FALSE.
-    */
-    using_proxy = NO;
 
-    if (!strcasecomp(acc_method, "news")) {
-	/*
-	**  News is different, so we need to check the name of the server,
-	**  as well as the default port for selective exclusions.
-	*/
-	char *host = NULL;
-	if ((host = HTParse(addr, "", PARSE_HOST))) {
-	    if (strchr(host, ':') == NULL) {
+    if (!override_flag && !using_proxy) {   /* else ignore no_proxy env var */
+	if (!strcasecomp(acc_method, "news")) {
+	    /*
+	    **  News is different, so we need to check the name of the server,
+	    **  as well as the default port for selective exclusions.
+	    */
+	    char *host = NULL;
+	    if ((host = HTParse(addr, "", PARSE_HOST))) {
+		if (strchr(host, ':') == NULL) {
+		    StrAllocCopy(Server_addr, "news://");
+		    StrAllocCat(Server_addr, host);
+		    StrAllocCat(Server_addr, ":119/");
+		}
+		FREE(host);
+	    } else if (LYGetEnv("NNTPSERVER") != NULL) {
 		StrAllocCopy(Server_addr, "news://");
-		StrAllocCat(Server_addr, host);
+		StrAllocCat(Server_addr, LYGetEnv("NNTPSERVER"));
 		StrAllocCat(Server_addr, ":119/");
 	    }
-	    FREE(host);
-	} else if (getenv("NNTPSERVER") != NULL) {
-	    StrAllocCopy(Server_addr, "news://");
-	    StrAllocCat(Server_addr, (char *)getenv("NNTPSERVER"));
-	    StrAllocCat(Server_addr, ":119/");
-	 }
-    } else if (!strcasecomp(acc_method, "wais")) {
-	/*
-	**  Wais also needs checking of the default port
-	**  for selective exclusions.
-	*/
-	char *host = NULL;
-	if ((host = HTParse(addr, "", PARSE_HOST))) {
-	    if (!(strchr(host, ':'))) {
-		StrAllocCopy(Server_addr, "wais://");
-		StrAllocCat(Server_addr, host);
-		StrAllocCat(Server_addr, ":210/");
+	} else if (!strcasecomp(acc_method, "wais")) {
+	    /*
+	    **  Wais also needs checking of the default port
+	    **  for selective exclusions.
+	    */
+	    char *host = NULL;
+	    if ((host = HTParse(addr, "", PARSE_HOST))) {
+		if (!(strchr(host, ':'))) {
+		    StrAllocCopy(Server_addr, "wais://");
+		    StrAllocCat(Server_addr, host);
+		    StrAllocCat(Server_addr, ":210/");
+		}
+		FREE(host);
 	    }
-	    FREE(host);
-	}
-	else
+	    else
+		StrAllocCopy(Server_addr, addr);
+	} else {
 	    StrAllocCopy(Server_addr, addr);
-    } else {
-	StrAllocCopy(Server_addr, addr);
+	}
+	override_flag = override_proxy(Server_addr);
     }
 
-    if (!override_proxy(Server_addr)) {
-	char * gateway_parameter, *gateway, *proxy;
+    if (!override_flag && !using_proxy) {
+	char * gateway_parameter = NULL, *gateway, *proxy;
 
 	/*
 	**  Search for gateways.
 	*/
-	gateway_parameter = (char *)calloc(1, (strlen(acc_method) + 20));
-	if (gateway_parameter == NULL)
-	    outofmem(__FILE__, "HTLoad");
-	strcpy(gateway_parameter, "WWW_");
-	strcat(gateway_parameter, acc_method);
-	strcat(gateway_parameter, "_GATEWAY");
-	gateway = (char *)getenv(gateway_parameter); /* coerce for decstation */
+	HTSprintf0(&gateway_parameter, "WWW_%s_GATEWAY", acc_method);
+	gateway = LYGetEnv(gateway_parameter); /* coerce for decstation */
 
 	/*
 	**  Search for proxy servers.
@@ -477,17 +499,16 @@ PRIVATE int get_physical ARGS2(
 	    /*
 	    ** If we got to here, a file URL is for ftp on a remote host. - FM
 	    */
-	    strcpy(gateway_parameter, "ftp");
+	    strcpy(gateway_parameter, "ftp_proxy");
 	else
-	    strcpy(gateway_parameter, acc_method);
-	strcat(gateway_parameter, "_proxy");
-	proxy = (char *)getenv(gateway_parameter);
+	    sprintf(gateway_parameter, "%s_proxy", acc_method);
+	proxy = LYGetEnv(gateway_parameter);
 	FREE(gateway_parameter);
 
 	if (gateway)
-	    CTRACE(tfp, "Gateway found: %s\n", gateway);
+	    CTRACE((tfp, "Gateway found: %s\n", gateway));
 	if (proxy)
-	    CTRACE(tfp, "proxy server found: %s\n", proxy);
+	    CTRACE((tfp, "proxy server found: %s\n", proxy));
 
 	/*
 	**  Proxy servers have precedence over gateway servers.
@@ -495,6 +516,14 @@ PRIVATE int get_physical ARGS2(
 	if (proxy) {
 	    char * gatewayed = NULL;
 	    StrAllocCopy(gatewayed,proxy);
+	    if (!strncmp(gatewayed, "http", 4)) {
+		char *cp = strrchr(gatewayed, '/');
+		/* Append a slash to the proxy specification if it doesn't
+		 * end in one but otherwise looks normal (starts with "http",
+		 * has no '/' other than ones before the hostname). - kw */
+		if (cp && (cp - gatewayed) <= 7)
+		    LYAddHtmlSep(&gatewayed);
+	    }
 	    /*
 	    ** Ensure that the proxy server uses ftp for file URLs. - FM
 	    */
@@ -510,8 +539,8 @@ PRIVATE int get_physical ARGS2(
 	    FREE(gatewayed);
 	    FREE(acc_method);
 
-	    acc_method =  HTParse(HTAnchor_physical(anchor),
-		"http:", PARSE_ACCESS);
+	    acc_method = HTParse(HTAnchor_physical(anchor),
+				 STR_HTTP_URL, PARSE_ACCESS);
 
 	} else if (gateway) {
 	    char * path = HTParse(addr, "",
@@ -523,8 +552,8 @@ PRIVATE int get_physical ARGS2(
 	    FREE(gatewayed);
 	    FREE(acc_method);
 
-	    acc_method =  HTParse(HTAnchor_physical(anchor),
-		"http:", PARSE_ACCESS);
+	    acc_method = HTParse(HTAnchor_physical(anchor),
+				 STR_HTTP_URL, PARSE_ACCESS);
 	}
     }
     FREE(Server_addr);
@@ -533,6 +562,7 @@ PRIVATE int get_physical ARGS2(
     /*
     **	Search registered protocols to find suitable one.
     */
+    result = HT_NO_ACCESS;
     {
 	int i, n;
 #ifndef NO_INIT
@@ -544,13 +574,14 @@ PRIVATE int get_physical ARGS2(
 	    if (!strcmp(p->name, acc_method)) {
 		HTAnchor_setProtocol(anchor, p);
 		FREE(acc_method);
-		return (HT_OK);
+		result = HT_OK;
+		break;
 	    }
 	}
     }
 
     FREE(acc_method);
-    return HT_NO_ACCESS;
+    return result;
 }
 
 /*
@@ -562,9 +593,6 @@ PRIVATE int get_physical ARGS2(
  *  I want the "pop" to occur as soon as possible after loading
  *  has finished. - kw @@@
  */
-
-extern char*UCAssume_MIMEcharset;
-
 PUBLIC void LYUCPushAssumed ARGS1(
     HTParentAnchor *,	anchor)
 {
@@ -578,13 +606,16 @@ PUBLIC void LYUCPushAssumed ARGS1(
 	if (anchor_UCI && anchor_UCI->MIMEname) {
 	    pushed_assume_MIMEname = UCAssume_MIMEcharset;
 	    UCAssume_MIMEcharset = NULL;
+	    if (HTCJK == JAPANESE)
+		StrAllocCopy(UCAssume_MIMEcharset, pushed_assume_MIMEname);
+	    else
 	    StrAllocCopy(UCAssume_MIMEcharset, anchor_UCI->MIMEname);
 	    pushed_assume_LYhndl = anchor_LYhndl;
 	    /* some diagnostics */
 	    if (UCLYhndl_for_unspec != anchor_LYhndl)
-	    CTRACE(tfp, "LYUCPushAssumed: UCLYhndl_for_unspec changed %d -> %d\n",
+	    CTRACE((tfp, "LYUCPushAssumed: UCLYhndl_for_unspec changed %d -> %d\n",
 			UCLYhndl_for_unspec,
-			anchor_LYhndl);
+			anchor_LYhndl));
 	    UCLYhndl_for_unspec = anchor_LYhndl;
 	    return;
 	}
@@ -603,9 +634,9 @@ PUBLIC int LYUCPopAssumed NOARGS
     if (pushed_assume_LYhndl >= 0) {
 	/* some diagnostics */
 	if (UCLYhndl_for_unspec != pushed_assume_LYhndl)
-	CTRACE(tfp, "LYUCPopAssumed: UCLYhndl_for_unspec changed %d -> %d\n",
+	CTRACE((tfp, "LYUCPopAssumed: UCLYhndl_for_unspec changed %d -> %d\n",
 		    UCLYhndl_for_unspec,
-		    pushed_assume_LYhndl);
+		    pushed_assume_LYhndl));
 	UCLYhndl_for_unspec = pushed_assume_LYhndl;
 	pushed_assume_LYhndl = -1;
 	FREE(UCAssume_MIMEcharset);
@@ -641,16 +672,22 @@ PRIVATE int HTLoad ARGS4(
     HTProtocol *p;
     int status = get_physical(addr, anchor);
     if (status == HT_FORBIDDEN) {
+	 /* prevent crash if telnet or similar was forbidden by rule. - kw */
+	LYFixCursesOn("show alert:");
 	return HTLoadError(sink, 500, gettext("Access forbidden by rule"));
+    } else if (status == HT_REDIRECTING) {
+	return status;	/* fake redirection by rule, to redirecting_url */
     }
     if (status < 0)
 	return status;	/* Can't resolve or forbidden */
 
+    /* prevent crash if telnet or similar mapped or proxied by rule. - kw */
+    LYFixCursesOnForAccess(addr, HTAnchor_physical(anchor));
     p = (HTProtocol *)HTAnchor_protocol(anchor);
-    anchor->underway = TRUE;		/* Hack to deal with caching */
-    status= (*(p->load))(HTAnchor_physical(anchor),
+    anchor->parent->underway = TRUE;		/* Hack to deal with caching */
+    status= p->load(HTAnchor_physical(anchor),
 			anchor, format_out, sink);
-    anchor->underway = FALSE;
+    anchor->parent->underway = FALSE;
     LYUCPopAssumed();
     return status;
 }
@@ -665,16 +702,18 @@ PUBLIC HTStream *HTSaveStream ARGS1(
     if (!p)
 	return NULL;
 
-    return (*p->saveStream)(anchor);
+    return p->saveStream(anchor);
 }
 
+PUBLIC int redirection_attempts = 0; /* counter in HTLoadDocument */
+
 /*	Load a document - with logging etc		HTLoadDocument()
 **	----------------------------------
 **
 **	- Checks or documents already loaded
 **	- Logs the access
 **	- Allows stdin filter option
-**	- Trace ouput and error messages
+**	- Trace output and error messages
 **
 **  On Entry,
 **	  anchor	    is the node_anchor for the document
@@ -687,19 +726,18 @@ PUBLIC HTStream *HTSaveStream ARGS1(
 */
 
 PRIVATE BOOL HTLoadDocument ARGS4(
-	CONST char *,		full_address,
+	CONST char *,		full_address, /* may include #fragment */
 	HTParentAnchor *,	anchor,
 	HTFormat,		format_out,
 	HTStream*,		sink)
 {
-    int		status;
-    HText *	text;
+    int     status;
+    HText * text;
     CONST char * address_to_load = full_address;
     char *cp;
     BOOL ForcingNoCache = LYforce_no_cache;
-    static int redirection_attempts = 0;
 
-    CTRACE (tfp, "HTAccess: loading document %s\n", address_to_load);
+    CTRACE((tfp, "HTAccess: loading document %s\n", address_to_load));
 
     /*
     **	Free use_this_url_instead and reset permanent_redirection
@@ -734,10 +772,10 @@ PRIVATE BOOL HTLoadDocument ARGS4(
      *	are LYNXIMGMAP documents, for which we defer to LYLoadIMGmap
      *	for prompting if necessary. - kw
      */
-    if (LYinternal_flag && !LYforce_no_cache &&
+    text = (HText *)HTAnchor_document(anchor);
+    if (LYinternal_flag && !text && !LYforce_no_cache &&
 	anchor->post_data && !anchor->safe &&
-	(text = (HText *)HTAnchor_document(anchor)) == NULL &&
-	strncmp(full_address, "LYNXIMGMAP:", 11) &&
+	!isLYNXIMGMAP(full_address) &&
 	HTConfirm(gettext("Document with POST content not found in cache.  Resubmit?"))
 	!= TRUE) {
 	return NO;
@@ -755,9 +793,9 @@ PRIVATE BOOL HTLoadDocument ARGS4(
 	       !strncmp(cp, "Location=", 9)) {
 	    DocAddress NewDoc;
 
-	    CTRACE (tfp, "HTAccess: '%s' is a redirection URL.\n",
-			  anchor->address);
-	    CTRACE (tfp, "HTAccess: Redirecting to '%s'\n", cp+9);
+	    CTRACE((tfp, "HTAccess: '%s' is a redirection URL.\n",
+			  anchor->address));
+	    CTRACE((tfp, "HTAccess: Redirecting to '%s'\n", cp+9));
 
 	    /*
 	    **	Don't exceed the redirection_attempts limit. - FM
@@ -779,7 +817,7 @@ PRIVATE BOOL HTLoadDocument ARGS4(
 	    NewDoc.bookmark = anchor->bookmark;
 	    NewDoc.isHEAD = anchor->isHEAD;
 	    NewDoc.safe = anchor->safe;
-	    anchor = (HTParentAnchor *)HTAnchor_findAddress(&NewDoc);
+	    anchor = HTAnchor_findAddress(&NewDoc);
 	}
     }
     /*
@@ -794,7 +832,8 @@ PRIVATE BOOL HTLoadDocument ARGS4(
     /*
     **	See if we can use an already loaded document.
     */
-    if (!LYforce_no_cache && (text = (HText *)HTAnchor_document(anchor))) {
+    text = (HText *)HTAnchor_document(anchor);
+    if (text && !LYforce_no_cache) {
 	/*
 	**  We have a cached rendition of the target document.
 	**  Check if it's OK to re-use it.  We consider it OK if:
@@ -846,16 +885,16 @@ PRIVATE BOOL HTLoadDocument ARGS4(
 	**  etc.) but the code for doing those other things isn't
 	**  available yet.
 	*/
-#ifdef DONT_TRACK_INTERNAL_LINKS
-	if (LYoverride_no_cache || !HText_hasNoCacheSet(text) ||
-	    !HText_AreDifferent(anchor, full_address))
-#else
 	if (LYoverride_no_cache ||
+#ifdef DONT_TRACK_INTERNAL_LINKS
+	    !HText_hasNoCacheSet(text) ||
+	    !HText_AreDifferent(anchor, full_address)
+#else
 	    ((LYinternal_flag || !HText_hasNoCacheSet(text)) &&
-	     strncmp(full_address, "LYNXIMGMAP:", 11)))
+	     !isLYNXIMGMAP(full_address))
 #endif /* TRACK_INTERNAL_LINKS */
-	{
-	    CTRACE(tfp, "HTAccess: Document already in memory.\n");
+	) {
+	    CTRACE((tfp, "HTAccess: Document already in memory.\n"));
 	    HText_select(text);
 
 #ifdef DIRED_SUPPORT
@@ -866,10 +905,18 @@ PRIVATE BOOL HTLoadDocument ARGS4(
 	    return YES;
 	} else {
 	    ForcingNoCache = YES;
-	    CTRACE(tfp, "HTAccess: Auto-reloading document.\n");
+	    CTRACE((tfp, "HTAccess: Auto-reloading document.\n"));
 	}
     }
 
+    if (text && HText_HaveUserChangedForms(text)) {
+	/*
+	 * Issue a warning.  User forms content will be lost.
+	 * Will not restore changed forms, currently.
+	 */
+	HTAlert(RELOADING_FORM);
+    }
+
     /*
     **	Get the document from the net.	If we are auto-reloading,
     **	the mutable anchor elements from the previous rendition
@@ -878,10 +925,28 @@ PRIVATE BOOL HTLoadDocument ARGS4(
     */
     LYforce_no_cache = NO;  /* reset after each time through */
     if (ForcingNoCache) {
-	FREE(anchor->title);
+	FREE(anchor->title);  /* ??? */
     }
     status = HTLoad(address_to_load, anchor, format_out, sink);
-    CTRACE(tfp, "HTAccess:  status=%d\n", status);
+    CTRACE((tfp, "HTAccess:  status=%d\n", status));
+
+    /*
+     *  RECOVERY:
+     *  if the loading failed, and we had a cached HText copy,
+     *  and no new HText created - use a previous copy, issue a warning.
+     */
+    if (text && status < 0 && (HText *)HTAnchor_document(anchor) == text) {
+	HTAlert(gettext("Loading failed, use a previous copy."));
+	CTRACE((tfp, "HTAccess: Loading failed, use a previous copy.\n"));
+	HText_select(text);
+
+#ifdef DIRED_SUPPORT
+	if (HTAnchor_format(anchor) == WWW_DIRED)
+	    lynx_edit_mode = TRUE;
+#endif
+	redirection_attempts = 0;
+	return YES;
+    }
 
     /*
     **	Log the access if necessary.
@@ -895,11 +960,11 @@ PRIVATE BOOL HTLoadDocument ARGS4(
 		status < 0 ? "FAIL" : "GET",
 		full_address);
 	fflush(HTlogfile);	/* Actually update it on disk */
-	CTRACE(tfp, "Log: %24.24s %s %s %s\n",
+	CTRACE((tfp, "Log: %24.24s %s %s %s\n",
 		    ctime(&theTime),
 		    HTClientHost ? HTClientHost : "local",
 		    status < 0 ? "FAIL" : "GET",
-		    full_address);
+		    full_address));
     }
 
     /*
@@ -923,11 +988,18 @@ PRIVATE BOOL HTLoadDocument ARGS4(
 	**  So, instead, we'll go all the way back to the top of getfile
 	**  in LYGetFile.c when the status is HT_REDIRECTING.  This may
 	**  seem bizarre, but it works like a charm! - FM
+	**
+	**  Actually, the location header for redirections is now again
+	**  picked up in HTMIME.c.  But that's an internal matter between
+	**  HTTP.c and HTMIME.c, is still under control of HTLoadHTTP for
+	**  http URLs, is done in a way that doesn't load the redirection
+	**  response's body (except when wanted as an error fallback), and
+	**  thus need not concern us here. - kw 1999-12-02
 	*/
-	CTRACE(tfp, "HTAccess: '%s' is a redirection URL.\n",
-		    address_to_load);
-	CTRACE(tfp, "HTAccess: Redirecting to '%s'\n",
-		     redirecting_url);
+	CTRACE((tfp, "HTAccess: '%s' is a redirection URL.\n",
+		    address_to_load));
+	CTRACE((tfp, "HTAccess: Redirecting to '%s'\n",
+		     redirecting_url));
 	/*
 	**  Prevent circular references.
 	*/
@@ -967,32 +1039,32 @@ PRIVATE BOOL HTLoadDocument ARGS4(
     permanent_redirection = FALSE;
 
     if (status == HT_LOADED) {
-	CTRACE(tfp, "HTAccess: `%s' has been accessed.\n",
-		    full_address);
+	CTRACE((tfp, "HTAccess: `%s' has been accessed.\n",
+		    full_address));
 	return YES;
     }
     if (status == HT_PARTIAL_CONTENT) {
 	HTAlert(gettext("Loading incomplete."));
-	CTRACE(tfp, "HTAccess: `%s' has been accessed, partial content.\n",
-		    full_address);
+	CTRACE((tfp, "HTAccess: `%s' has been accessed, partial content.\n",
+		    full_address));
 	return YES;
     }
 
     if (status == HT_NO_DATA) {
-	CTRACE(tfp, "HTAccess: `%s' has been accessed, No data left.\n",
-		    full_address);
+	CTRACE((tfp, "HTAccess: `%s' has been accessed, No data left.\n",
+		    full_address));
 	return NO;
     }
 
     if (status == HT_NOT_LOADED) {
-	CTRACE(tfp, "HTAccess: `%s' has been accessed, No data loaded.\n",
-		    full_address);
+	CTRACE((tfp, "HTAccess: `%s' has been accessed, No data loaded.\n",
+		    full_address));
 	return NO;
     }
 
     if (status == HT_INTERRUPTED) {
-	CTRACE(tfp, "HTAccess: `%s' has been accessed, transfer interrupted.\n",
-		    full_address);
+	CTRACE((tfp, "HTAccess: `%s' has been accessed, transfer interrupted.\n",
+		    full_address));
 	return NO;
     }
 
@@ -1004,9 +1076,9 @@ PRIVATE BOOL HTLoadDocument ARGS4(
 	fprintf(stderr,
  gettext("**** HTAccess: socket or file number returned by obsolete load routine!\n"));
 	fprintf(stderr,
- gettext("**** HTAccess: Internal software error.  Please mail lynx_dev@sig.net!\n"));
+ gettext("**** HTAccess: Internal software error.  Please mail lynx-dev@sig.net!\n"));
 	fprintf(stderr, gettext("**** HTAccess: Status returned was: %d\n"),status);
-	exit(-1);
+	exit(EXIT_FAILURE);
     }
 
     /* Failure in accessing a document */
@@ -1018,7 +1090,7 @@ PRIVATE BOOL HTLoadDocument ARGS4(
     _HTProgress(cp);
     FREE(cp);
 
-    CTRACE(tfp, "HTAccess: Can't access `%s'\n", full_address);
+    CTRACE((tfp, "HTAccess: Can't access `%s'\n", full_address));
     HTLoadError(sink, 500, gettext("Unable to access document."));
     return NO;
 } /* HTLoadDocument */
@@ -1038,7 +1110,7 @@ PUBLIC BOOL HTLoadAbsolute ARGS1(
 	CONST DocAddress *,	docaddr)
 {
     return HTLoadDocument(docaddr->address,
-			  HTAnchor_parent(HTAnchor_findAddress(docaddr)),
+			  HTAnchor_findAddress(docaddr),
 			  (HTOutputFormat ? HTOutputFormat : WWW_PRESENT),
 			  HTOutputStream);
 }
@@ -1061,7 +1133,7 @@ PUBLIC BOOL HTLoadToStream ARGS3(
 	HTStream *,	sink)
 {
     return HTLoadDocument(addr,
-			  HTAnchor_parent(HTAnchor_findAddress(addr)),
+			  HTAnchor_findSimpleAddress(addr),
 			  (HTOutputFormat ? HTOutputFormat : WWW_PRESENT),
 			  sink);
 }
@@ -1086,7 +1158,6 @@ PUBLIC BOOL HTLoadRelative ARGS2(
     BOOL result;
     char * mycopy = NULL;
     char * stripped = NULL;
-    char * current_address = HTAnchor_address((HTAnchor*)here);
 
     full_address.address = NULL;
     full_address.post_data = NULL;
@@ -1100,8 +1171,8 @@ PUBLIC BOOL HTLoadRelative ARGS2(
     stripped = HTStrip(mycopy);
     full_address.address =
 		HTParse(stripped,
-			current_address,
-			PARSE_ACCESS|PARSE_HOST|PARSE_PATH|PARSE_PUNCTUATION);
+			here->address,
+			PARSE_ALL_WITHOUT_ANCHOR);
     result = HTLoadAbsolute(&full_address);
     /*
     **	If we got redirection, result will be NO, but use_this_url_instead
@@ -1109,7 +1180,6 @@ PUBLIC BOOL HTLoadRelative ARGS2(
     **	is appropriate. - FM
     */
     FREE(full_address.address);
-    FREE(current_address);
     FREE(mycopy);  /* Memory leak fixed 10/7/92 -- JFG */
     return result;
 }
@@ -1137,14 +1207,12 @@ PUBLIC BOOL HTLoadAnchor ARGS1(
     if (HTAnchor_document(parent) == NULL) {	/* If not already loaded */
 						/* TBL 921202 */
 	BOOL result;
-	char * address = HTAnchor_address((HTAnchor*) parent);
 
-	result = HTLoadDocument(address,
+	result = HTLoadDocument(parent->address,
 				parent,
 				HTOutputFormat ?
 				HTOutputFormat : WWW_PRESENT,
 				HTOutputStream);
-	FREE(address);
 	if (!result) return NO;
 	loaded = YES;
     }
@@ -1152,7 +1220,9 @@ PUBLIC BOOL HTLoadAnchor ARGS1(
     {
 	HText *text = (HText*)HTAnchor_document(parent);
 
-	if (destination != (HTAnchor *)parent) {  /* If child anchor */
+	if ((destination != (HTAnchor *)parent) &&
+	    (destination != (HTAnchor *)(parent->parent))) {
+						  /* If child anchor */
 	    HText_selectAnchor(text,		  /* Double display? @@ */
 			       (HTChildAnchor*)destination);
 	} else {
@@ -1193,7 +1263,7 @@ PUBLIC BOOL HTSearch ARGS2(
     CONST char * p, *s, *e;		/* Pointers into keywords */
     char * address = NULL;
     BOOL result;
-    char * escaped = (char *)calloc(1, ((strlen(keywords)*3) + 1));
+    char * escaped = typecallocn(char, (strlen(keywords)*3) + 1);
     static CONST BOOL isAcceptable[96] =
 
     /*	 0 1 2 3 4 5 6 7 8 9 A B C D E F */
@@ -1217,12 +1287,12 @@ PUBLIC BOOL HTSearch ARGS2(
     for (e = s + strlen(s); e > s && WHITE(*(e-1)); e--) /* Scan */
 	;	/* Skip trailers */
     for (q = escaped, p = s; p < e; p++) {	/* Scan stripped field */
-	unsigned char c = (unsigned char)TOASCII(*p);
+	unsigned char c = UCH(TOASCII(*p));
 	if (WHITE(*p)) {
 	    *q++ = '+';
 	} else if (HTCJK != NOCJK) {
 	    *q++ = *p;
-	} else if (c>=32 && c<=(unsigned char)127 && isAcceptable[c-32]) {
+	} else if (c>=32 && c<=UCH(127) && isAcceptable[c-32]) {
 	    *q++ = *p;				/* 930706 TBL for MVS bug */
 	} else {
 	    *q++ = '%';
@@ -1276,7 +1346,7 @@ PUBLIC BOOL HTSearchAbsolute ARGS2(
     abs_doc.isHEAD = FALSE;
     abs_doc.safe = FALSE;
 
-    anchor = (HTParentAnchor*)HTAnchor_findAddress(&abs_doc);
+    anchor = HTAnchor_findAddress(&abs_doc);
     return HTSearch(keywords, anchor);
 }
 
@@ -1298,7 +1368,7 @@ PUBLIC BOOL HTSearchAbsolute ARGS2(
 PUBLIC HTParentAnchor * HTHomeAnchor NOARGS
 {
     char * my_home_document = NULL;
-    char * home = (char *)getenv(LOGICAL_DEFAULT);
+    char * home = LYGetEnv(LOGICAL_DEFAULT);
     char * ref;
     HTParentAnchor * anchor;
 
@@ -1312,7 +1382,7 @@ PUBLIC HTParentAnchor * HTHomeAnchor NOARGS
 	FILE * fp = fopen(REMOTE_POINTER, "r");
 	char * status;
 	if (fp) {
-	    my_home_document = (char*)calloc(1, MAX_FILE_NAME);
+	    my_home_document = typecallocn(char, MAX_FILE_NAME);
 	    if (my_home_document == NULL)
 		outofmem(__FILE__, "HTHomeAnchor");
 	    status = fgets(my_home_document, MAX_FILE_NAME, fp);
@@ -1325,11 +1395,11 @@ PUBLIC HTParentAnchor * HTHomeAnchor NOARGS
 	    StrAllocCopy(my_home_document, REMOTE_ADDRESS);
     }
 
-#ifdef unix
+#ifdef UNIX
     if (my_home_document == NULL) {
 	FILE * fp = NULL;
-	CONST char * home =  (CONST char*)getenv("HOME");
-	if (home != null) {
+	char * home = LYGetEnv("HOME");
+	if (home != 0) {
 	    HTSprintf0(&my_home_document, "%s/%s", home, PERSONAL_DEFAULT);
 	    fp = fopen(my_home_document, "r");
 	}
@@ -1341,23 +1411,23 @@ PUBLIC HTParentAnchor * HTHomeAnchor NOARGS
 	if (fp) {
 	    fclose(fp);
 	} else {
-	    CTRACE(tfp, "HTBrowse: No local home document ~/%s or %s\n",
-			PERSONAL_DEFAULT, LOCAL_DEFAULT_FILE);
+	    CTRACE((tfp, "HTBrowse: No local home document ~/%s or %s\n",
+			PERSONAL_DEFAULT, LOCAL_DEFAULT_FILE));
 	    FREE(my_home_document);
 	}
     }
-#endif /* unix */
+#endif /* UNIX */
     ref = HTParse((my_home_document ?
 		   my_home_document : (HTClientHost ?
 				     REMOTE_ADDRESS : LAST_RESORT)),
-		  "file:",
-		  PARSE_ACCESS|PARSE_HOST|PARSE_PATH|PARSE_PUNCTUATION);
+		  STR_FILE_URL,
+		  PARSE_ALL_WITHOUT_ANCHOR);
     if (my_home_document) {
-	CTRACE(tfp, "HTAccess: Using custom home page %s i.e., address %s\n",
-		    my_home_document, ref);
+	CTRACE((tfp, "HTAccess: Using custom home page %s i.e., address %s\n",
+		    my_home_document, ref));
 	FREE(my_home_document);
     }
-    anchor = (HTParentAnchor*)HTAnchor_findAddress(ref);
+    anchor = HTAnchor_findSimpleAddress(ref);
     FREE(ref);
     return anchor;
 }
