@@ -16,8 +16,12 @@
 **	returns		WWW file specification
 **
 */
-char * HTDOS_wwwName PARAMS((char * dosname));
+char * HTDOS_wwwName PARAMS((CONST char * dosname));
 
+/*
+ * Converts Unix slashes to DOS
+ */
+char * HTDOS_slashes PARAMS((char * path));
 
 /* PUBLIC                                                       HTDOS_name()
 **              CONVERTS WWW name into a DOS name
@@ -31,5 +35,10 @@ char * HTDOS_wwwName PARAMS((char * dosname));
 */
 char * HTDOS_name PARAMS((char * wwwname));
 
+#ifdef WIN_EX 
+char * HTDOS_short_name (char * fn);
+#else 
+#define HTDOS_short_name(fn)  fn 
+#endif
 
 #endif /*  HTDOS_H */
