@@ -21,18 +21,18 @@
 extern int HTfileSortMethod;  /* specifies the method of sorting */
 
 
-/* PUBLIC						 HTMake_VMS_name()
+/* PUBLIC						 HTVMS_name()
 **		CONVERTS WWW name into a VMS name
 ** ON ENTRY:
 **	nn		Node Name (optional)
 **	fn		WWW file name
 **
 ** ON EXIT:
-**	returns 	vms file specification
+**	returns		vms file specification
 **
 ** Bug:	Returns pointer to static -- non-reentrant
 */
-PUBLIC char * HTMake_VMS_name PARAMS((
+PUBLIC char * HTVMS_name PARAMS((
 	CONST char * nn,
 	CONST char * fn));
 
@@ -68,7 +68,3 @@ Return Host Name
 extern CONST char * HTHostName NOPARAMS;
 
 #endif
-
-/*
-
-   end  */
