@@ -20,14 +20,14 @@ PUBLIC float HTMaxSecs = 1e10;		/* No effective limit */
 PUBLIC float HTMaxLength = 1e10;	/* No effective limit */
 PUBLIC long int HTMaxBytes  = 0;	/* No effective limit */
 
-#ifdef unix
+#ifdef UNIX
 #ifdef NeXT
 #define PRESENT_POSTSCRIPT "open %s; /bin/rm -f %s\n"
 #else
 #define PRESENT_POSTSCRIPT "(ghostview %s ; /bin/rm -f %s)&\n"
 			   /* Full pathname would be better! */
 #endif /* NeXT */
-#endif /* unix */
+#endif /* UNIX */
 
 #include <HTML.h>
 #include <HTMLDTD.h>
@@ -36,6 +36,7 @@ PUBLIC long int HTMaxBytes  = 0;	/* No effective limit */
 #include <HTList.h>
 #include <HTInit.h>
 #include <HTTCP.h>
+#include <HTTP.h>
 /*	Streams and structured streams which we use:
 */
 #include <HTFWriter.h>
@@ -49,8 +50,11 @@ PUBLIC long int HTMaxBytes  = 0;	/* No effective limit */
 #include <LYGlobalDefs.h>
 #include <LYLeaks.h>
 
+#ifdef DISP_PARTIAL
+#include <LYMainLoop.h>
+#endif
+
 PUBLIC	BOOL HTOutputSource = NO;	/* Flag: shortcut parser to stdout */
-/* extern  BOOL interactive; LJM */
 
 #ifdef ORIGINAL
 struct _HTStream {
@@ -85,21 +89,21 @@ PRIVATE void HTFreePresentations NOPARAMS;
 PUBLIC void HTSetPresentation ARGS6(
 	CONST char *,	representation,
 	CONST char *,	command,
-	float,		quality,
-	float,		secs,
-	float,		secs_per_byte,
+	double,		quality,
+	double,		secs,
+	double,		secs_per_byte,
 	long int,	maxbytes)
 {
-    HTPresentation * pres = (HTPresentation *)malloc(sizeof(HTPresentation));
+    HTPresentation * pres = typecalloc(HTPresentation);
     if (pres == NULL)
 	outofmem(__FILE__, "HTSetPresentation");
 
     pres->rep = HTAtom_for(representation);
     pres->rep_out = WWW_PRESENT;		/* Fixed for now ... :-) */
     pres->converter = HTSaveAndExecute;		/* Fixed for now ...	 */
-    pres->quality = quality;
-    pres->secs = secs;
-    pres->secs_per_byte = secs_per_byte;
+    pres->quality = (float) quality;
+    pres->secs = (float) secs;
+    pres->secs_per_byte = (float) secs_per_byte;
     pres->maxbytes = maxbytes;
     pres->command = NULL;
     StrAllocCopy(pres->command, command);
@@ -135,7 +139,7 @@ PUBLIC void HTSetConversion ARGS7(
 	float,		secs_per_byte,
 	long int,	maxbytes)
 {
-    HTPresentation * pres = (HTPresentation *)malloc(sizeof(HTPresentation));
+    HTPresentation * pres = typecalloc(HTPresentation);
     if (pres == NULL)
 	outofmem(__FILE__, "HTSetConversion");
 
@@ -237,11 +241,11 @@ PUBLIC int HTGetCharacter NOARGS
 		if (status == 0)
 		    return EOF;
 		if (status == HT_INTERRUPTED) {
-		    CTRACE(tfp, "HTFormat: Interrupted in HTGetCharacter\n");
+		    CTRACE((tfp, "HTFormat: Interrupted in HTGetCharacter\n"));
 		    interrupted_in_htgetcharacter = 1;
 		    return EOF;
 		}
-		CTRACE(tfp, "HTFormat: File read error %d\n", status);
+		CTRACE((tfp, "HTFormat: File read error %d\n", status));
 		return EOF; /* -1 is returned by UCX
 			       at end of HTTP link */
 	    }
@@ -251,22 +255,55 @@ PUBLIC int HTGetCharacter NOARGS
 	ch = *input_pointer++;
     } while (ch == (char) 13); /* Ignore ASCII carriage return */
 
-    return FROMASCII((unsigned char)ch);
+    return FROMASCII(UCH(ch));
 }
 
+#ifdef USE_SSL
+PUBLIC char HTGetSSLCharacter ARGS1(void *, handle)
+{
+    char ch;
+    interrupted_in_htgetcharacter = 0;
+    if(!handle)
+	return (char)EOF;
+    do {
+	if (input_pointer >= input_limit) {
+	    int status = SSL_read((SSL *)handle,
+				 input_buffer, INPUT_BUFFER_SIZE);
+	    if (status <= 0) {
+		if (status == 0)
+		    return (char)EOF;
+		if (status == HT_INTERRUPTED) {
+		    CTRACE((tfp, "HTFormat: Interrupted in HTGetSSLCharacter\n"));
+		    interrupted_in_htgetcharacter = 1;
+		    return (char)EOF;
+		}
+		CTRACE((tfp, "HTFormat: SSL_read error %d\n", status));
+		return (char)EOF; /* -1 is returned by UCX
+				     at end of HTTP link */
+	    }
+	    input_pointer = input_buffer;
+	    input_limit = input_buffer + status;
+	}
+	ch = *input_pointer++;
+    } while (ch == (char) 13); /* Ignore ASCII carriage return */
+
+    return FROMASCII(ch);
+}
+#endif /* USE_SSL */
+
 /*  Match maintype to any MIME type starting with maintype,
  *  for example:  image/gif should match image
  */
 PRIVATE int half_match ARGS2(char *,trial_type, char *,target)
 {
-    char *cp=strchr(trial_type,'/');
+    char *cp = strchr(trial_type, '/');
 
     /* if no '/' or no '*' */
     if (!cp || *(cp+1) != '*')
 	return 0;
 
-    CTRACE(tfp, "HTFormat: comparing %s and %s for half match\n",
-		trial_type, target);
+    CTRACE((tfp, "HTFormat: comparing %s and %s for half match\n",
+		trial_type, target));
 
 	/* main type matches */
     if (!strncmp(trial_type, target, (cp-trial_type)-1))
@@ -275,6 +312,8 @@ PRIVATE int half_match ARGS2(char *,trial_type, char *,target)
     return 0;
 }
 
+#define WWW_WILDCARD_REP_OUT HTAtom_for("*")
+
 /*		Look up a presentation
 **		----------------------
 **
@@ -291,10 +330,10 @@ PRIVATE HTPresentation * HTFindPresentation ARGS3(
 	HTFormat,		rep_out,
 	HTPresentation*,	fill_in)
 {
-    HTAtom * wildcard = HTAtom_for("*");
+    HTAtom * wildcard = NULL; /* = HTAtom_for("*"); lookup when needed - kw */
 
-    CTRACE(tfp, "HTFormat: Looking up presentation for %s to %s\n",
-		HTAtom_name(rep_in), HTAtom_name(rep_out));
+    CTRACE((tfp, "HTFormat: Looking up presentation for %s to %s\n",
+		HTAtom_name(rep_in), HTAtom_name(rep_out)));
 
     /* don't do anymore do it in the Lynx code at startup LJM */
     /* if (!HTPresentations) HTFormatInit(); */ /* set up the list */
@@ -312,18 +351,21 @@ PRIVATE HTPresentation * HTFindPresentation ARGS3(
 	    pres = (HTPresentation *)HTList_objectAt(HTPresentations, i);
 	    if (pres->rep == rep_in) {
 		if (pres->rep_out == rep_out) {
-		    CTRACE(tfp, "FindPresentation: found exact match: %s\n",
-				HTAtom_name(pres->rep));
+		    CTRACE((tfp, "FindPresentation: found exact match: %s\n",
+				HTAtom_name(pres->rep)));
 		    return pres;
 
 		} else if (!fill_in) {
 		    continue;
-		} else if (pres->rep_out == wildcard) {
-		    if (!strong_wildcard_match)
-			strong_wildcard_match = pres;
-		    /* otherwise use the first one */
-		    CTRACE(tfp, "StreamStack: found strong wildcard match: %s\n",
-				HTAtom_name(pres->rep));
+		} else {
+		    if (!wildcard) wildcard = WWW_WILDCARD_REP_OUT;
+		    if (pres->rep_out == wildcard) {
+			if (!strong_wildcard_match)
+			    strong_wildcard_match = pres;
+			/* otherwise use the first one */
+			CTRACE((tfp, "StreamStack: found strong wildcard match: %s\n",
+				    HTAtom_name(pres->rep)));
+		    }
 		}
 
 	    } else if (!fill_in) {
@@ -335,8 +377,8 @@ PRIVATE HTPresentation * HTFindPresentation ARGS3(
 		    if (!strong_subtype_wildcard_match)
 			strong_subtype_wildcard_match = pres;
 		    /* otherwise use the first one */
-		    CTRACE(tfp, "StreamStack: found strong subtype wildcard match: %s\n",
-				HTAtom_name(pres->rep));
+		    CTRACE((tfp, "StreamStack: found strong subtype wildcard match: %s\n",
+				HTAtom_name(pres->rep)));
 		}
 	    }
 
@@ -345,11 +387,12 @@ PRIVATE HTPresentation * HTFindPresentation ARGS3(
 		    if (!weak_wildcard_match)
 			weak_wildcard_match = pres;
 		    /* otherwise use the first one */
-		    CTRACE(tfp, "StreamStack: found weak wildcard match: %s\n",
-				HTAtom_name(pres->rep_out));
-		}
-		if (pres->rep_out == wildcard) {
-		    if (!last_default_match)
+		    CTRACE((tfp, "StreamStack: found weak wildcard match: %s\n",
+				HTAtom_name(pres->rep_out)));
+
+		} else if (!last_default_match) {
+		    if (!wildcard) wildcard = WWW_WILDCARD_REP_OUT;
+		    if (pres->rep_out == wildcard)
 			 last_default_match = pres;
 		    /* otherwise use the first one */
 		}
@@ -390,30 +433,44 @@ PUBLIC HTStream * HTStreamStack ARGS4(
 {
     HTPresentation temp;
     HTPresentation *match;
+    HTStream *result;
 
-    CTRACE(tfp, "HTFormat: Constructing stream stack for %s to %s\n",
-		HTAtom_name(rep_in), HTAtom_name(rep_out));
+    CTRACE((tfp, "HTFormat: Constructing stream stack for %s to %s\n",
+		HTAtom_name(rep_in), HTAtom_name(rep_out)));
 
     /* don't return on WWW_SOURCE some people might like
      * to make use of the source!!!!  LJM
-     *//*
+     */
+#if 0
     if (rep_out == WWW_SOURCE || rep_out == rep_in)
-	return sink;  LJM */
+	return sink;	/*  LJM */
+#endif
 
-    if (rep_out == rep_in)
-	return sink;
+    if (rep_out == rep_in) {
+	result = sink;
 
-    if ((match = HTFindPresentation(rep_in, rep_out, &temp))) {
+    } else if ((match = HTFindPresentation(rep_in, rep_out, &temp))) {
 	if (match == &temp) {
-	    CTRACE(tfp, "StreamStack: Using %s\n", HTAtom_name(temp.rep_out));
+	    CTRACE((tfp, "StreamStack: Using %s\n", HTAtom_name(temp.rep_out)));
 	} else {
-	    CTRACE(tfp, "StreamStack: found exact match: %s\n",
-			HTAtom_name(match->rep));
+	    CTRACE((tfp, "StreamStack: found exact match: %s\n",
+			HTAtom_name(match->rep)));
 	}
-	return (*match->converter)(match, anchor, sink);
+	result = (*match->converter)(match, anchor, sink);
     } else {
-	return NULL;
+	result = NULL;
     }
+    if (TRACE) {
+	if (result && result->isa && result->isa->name) {
+	    CTRACE((tfp, "StreamStack: Returning \"%s\"\n", result->isa->name));
+	} else if (result) {
+	    CTRACE((tfp, "StreamStack: Returning *unknown* stream!\n"));
+	} else {
+	    CTRACE((tfp, "StreamStack: Returning NULL!\n"));
+	    CTRACE_FLUSH(tfp);	/* a crash may be imminent... - kw */
+	}
+    }
+    return result;
 }
 
 /*		Put a presentation near start of list
@@ -432,6 +489,56 @@ PUBLIC void HTReorderPresentation ARGS2(
 	HTList_addObject(HTPresentations, match);
     }
 }
+
+/*
+ * Setup 'get_accept' flag to denote presentations that are not redundant,
+ * and will be listed in "Accept:" header.
+ */
+PUBLIC void HTFilterPresentations NOARGS
+{
+    int i, j;
+    int n = HTList_count(HTPresentations);
+    HTPresentation *p, *q;
+    BOOL matched;
+    char *s, *t, *x, *y;
+
+    for (i = 0; i < n; i++) {
+	p = (HTPresentation *)HTList_objectAt(HTPresentations, i);
+	s = HTAtom_name(p->rep);
+
+	if (p->rep_out == WWW_PRESENT) {
+	    if (p->rep != WWW_SOURCE
+	     && strcasecomp(s, "www/mime")
+	     && strcasecomp(s, "www/compressed")
+	     && p->quality <= 1.0 && p->quality >= 0.0) {
+		for (j = 0, matched = FALSE; j < i; j++) {
+		    q = (HTPresentation *)HTList_objectAt(HTPresentations, j);
+		    t = HTAtom_name(q->rep);
+
+		    if (!strcasecomp(s, t)) {
+			matched = TRUE;
+			break;
+		    }
+		    if ((x = strchr(s, '/')) != 0
+		     && (y = strchr(t, '/')) != 0) {
+			int len1 = x++ - s;
+			int len2 = y++ - t;
+			int lens = (len1 > len2) ? len1 : len2;
+
+			if ((*t == '*' || !strncasecomp(s, t, lens))
+			 && (*y == '*' || !strcasecomp(x, y))) {
+			    matched = TRUE;
+			    break;
+			}
+		    }
+		}
+		if (!matched)
+		    p->get_accept = TRUE;
+	    }
+	}
+    }
+}
+
 /*		Find the cost of a filter stack
 **		-------------------------------
 **
@@ -446,10 +553,10 @@ PUBLIC float HTStackValue ARGS4(
 	float,			initial_value,
 	long int,		length)
 {
-    HTAtom * wildcard = HTAtom_for("*");
+    HTAtom * wildcard = WWW_WILDCARD_REP_OUT;
 
-    CTRACE(tfp, "HTFormat: Evaluating stream stack for %s worth %.3f to %s\n",
-		HTAtom_name(rep_in), initial_value, HTAtom_name(rep_out));
+    CTRACE((tfp, "HTFormat: Evaluating stream stack for %s worth %.3f to %s\n",
+		HTAtom_name(rep_in), initial_value, HTAtom_name(rep_out)));
 
     if (rep_out == WWW_SOURCE || rep_out == rep_in)
 	return 0.0;
@@ -474,7 +581,7 @@ PUBLIC float HTStackValue ARGS4(
 	}
     }
 
-    return -1e30;		/* Really bad */
+    return (float) -1e30;	/* Really bad */
 
 }
 
@@ -490,14 +597,13 @@ PUBLIC void HTDisplayPartial NOARGS
 #ifdef DISP_PARTIAL
     if (display_partial) {
 	/*
-	**  HText_getNumOfLines() = "current" number of lines received
+	**  HText_getNumOfLines() = "current" number of complete lines received
 	**  NumOfLines_partial = number of lines at the moment of last repaint.
+	**  (we update NumOfLines_partial only when we repaint the display.)
 	**
-	**  We update NumOfLines_partial only when we repaint the display.
-	**  -1 is the special value:
-	**  This is a synchronization flag switched to 0 when HText_new()
-	**  starts a new HTMainText object - all HText_ functions use it,
-	**  lines counter in particular [we call it from HText_getNumOfLines()].
+	**  display_partial could only be enabled in HText_new()
+	**  so a new HTMainText object available - all HText_ functions use it,
+	**  lines counter HText_getNumOfLines() in particular.
 	**
 	**  Otherwise HTMainText holds info from the previous document
 	**  and we may repaint it instead of the new one:
@@ -506,13 +612,13 @@ PUBLIC void HTDisplayPartial NOARGS
 	**
 	**  So repaint the page only when necessary:
 	*/
-	if ((NumOfLines_partial != -1)
-		/* new HText object available  */
-	&& ((Newline_partial + display_lines) > NumOfLines_partial)
+	int Newline_partial = LYGetNewline();
+
+	if (((Newline_partial + display_lines) - 1 > NumOfLines_partial)
 		/* current page not complete... */
 	&& (partial_threshold > 0 ?
-		((Newline_partial + partial_threshold) < HText_getNumOfLines()) :
-		((Newline_partial + display_lines) < HText_getNumOfLines()))
+		((Newline_partial + partial_threshold) -1 <= HText_getNumOfLines()) :
+		((Newline_partial + display_lines) - 1 <= HText_getNumOfLines()))
 		/*
 		 * Originally we rendered by increments of 2 lines,
 		 * but that got annoying on slow network connections.
@@ -520,8 +626,8 @@ PUBLIC void HTDisplayPartial NOARGS
 		 * If partial_threshold <= 0, then it's a full page
 		 */
 	) {
-	    NumOfLines_partial = HText_getNumOfLines();
-	    HText_pageDisplay(Newline_partial, "");
+	    if (LYMainLoop_pageDisplay(Newline_partial))
+		NumOfLines_partial = HText_getNumOfLines();
 	}
     }
 #else /* nothing */
@@ -532,24 +638,10 @@ PUBLIC void HTDisplayPartial NOARGS
 PUBLIC void HTFinishDisplayPartial NOARGS
 {
 #ifdef DISP_PARTIAL
-		    if (display_partial) {
-			/*
-			 *  Override Newline with a new value if user
-			 *  scrolled the document while downloading.
-			 */
-			if (Newline_partial != Newline
-			 && NumOfLines_partial > 0)
-			    Newline = Newline_partial;
-		    }
-
 		    /*
 		     *  End of incremental rendering stage here.
 		     */
 		    display_partial = FALSE;
-		    NumOfLines_partial = -1;       /* initialize to -1 */
-				/* -1 restrict HTDisplayPartial()   */
-				/* until HText_new() start next HTMainText */
-				/* and set the flag to 0  */
 #endif /* DISP_PARTIAL */
 }
 
@@ -592,6 +684,7 @@ PUBLIC int HTCopy ARGS4(
 	HTStream*,		sink)
 {
     HTStreamClass targetClass;
+    BOOL suppress_readprogress = NO;
     int bytes;
     int rv = 0;
 
@@ -624,7 +717,14 @@ PUBLIC int HTCopy ARGS4(
 	    goto finished;
 	}
 
+#ifdef USE_SSL
+	if (handle)
+	    status = SSL_read((SSL *)handle, input_buffer, INPUT_BUFFER_SIZE);
+	else
+	    status = NETREAD(file_number, input_buffer, INPUT_BUFFER_SIZE);
+#else
 	status = NETREAD(file_number, input_buffer, INPUT_BUFFER_SIZE);
+#endif /* USE_SSL */
 
 	if (status <= 0) {
 	    if (status == 0) {
@@ -638,6 +738,9 @@ PUBLIC int HTCopy ARGS4(
 		    rv = -1;
 		goto finished;
 	    } else if (SOCKET_ERRNO == ENOTCONN ||
+#ifdef _WINDOWS	/* 1997/11/10 (Mon) 16:57:18 */
+		       SOCKET_ERRNO == ETIMEDOUT ||
+#endif
 		       SOCKET_ERRNO == ECONNRESET ||
 		       SOCKET_ERRNO == EPIPE) {
 		/*
@@ -659,10 +762,10 @@ PUBLIC int HTCopy ARGS4(
 		    *  TCP stacks for VMS etc., so this is currently
 		    *  only for UNIX. - kw
 		    */
-		    HTInetStatus("NETREAD");
-		    HTAlert("Unexpected server disconnect.");
-		   CTRACE(tfp,
-	    "HTCopy: Unexpected server disconnect. Treating as completed.\n");
+		   HTInetStatus("NETREAD");
+		   HTAlert("Unexpected server disconnect.");
+		   CTRACE((tfp,
+	    "HTCopy: Unexpected server disconnect. Treating as completed.\n"));
 		   status = 0;
 		   break;
 #else  /* !UNIX */
@@ -670,8 +773,8 @@ PUBLIC int HTCopy ARGS4(
 		    *  Treat what we've gotten already
 		    *  as the complete transmission. - FM
 		    */
-		   CTRACE(tfp,
-	    "HTCopy: Unexpected server disconnect.  Treating as completed.\n");
+		   CTRACE((tfp,
+	    "HTCopy: Unexpected server disconnect.  Treating as completed.\n"));
 		   status = 0;
 		   break;
 #endif /* UNIX */
@@ -700,6 +803,16 @@ PUBLIC int HTCopy ARGS4(
 	    break;
 	}
 
+	/*
+	 *  Suppress ReadProgress messages when collecting a redirection
+	 *  message, at least initially (unless/until anchor->content_type
+	 *  gets changed, probably by the MIME message parser).  That way
+	 *  messages put up by the HTTP module or elsewhere can linger in
+	 *  the statusline for a while. - kw
+	 */
+	suppress_readprogress = (anchor && anchor->content_type &&
+				 !strcmp(anchor->content_type,
+					 "message/x-http-redirection"));
 #ifdef NOT_ASCII
 	{
 	    char * p;
@@ -711,7 +824,8 @@ PUBLIC int HTCopy ARGS4(
 
 	(*targetClass.put_block)(sink, input_buffer, status);
 	bytes += status;
-	HTReadProgress(bytes, anchor ? anchor->content_length : 0);
+	if (!suppress_readprogress)
+	    HTReadProgress(bytes, anchor ? anchor->content_length : 0);
 	HTDisplayPartial();
 
     } /* next bufferload */
@@ -767,8 +881,8 @@ PUBLIC int HTFileCopy ARGS2(
 		rv = HT_LOADED;
 		break;
 	    }
-	    CTRACE(tfp, "HTFormat: Read error, read returns %d\n",
-			ferror(fp));
+	    CTRACE((tfp, "HTFormat: Read error, read returns %d\n",
+			ferror(fp)));
 	    if (bytes) {
 		rv = HT_PARTIAL_CONTENT;
 	    } else {
@@ -780,7 +894,13 @@ PUBLIC int HTFileCopy ARGS2(
 	(*targetClass.put_block)(sink, input_buffer, status);
 	bytes += status;
 	HTReadProgress(bytes, 0);
-	HTDisplayPartial();
+	/*  Suppress last screen update in partial mode - a regular update
+	 *  under control of mainloop() should follow anyway. - kw
+	 */
+#ifdef DISP_PARTIAL
+	if (display_partial && bytes != HTMainAnchor->content_length)
+	    HTDisplayPartial();
+#endif
 
 	if (HTCheckForInterrupt()) {
 	    _HTProgress (TRANSFER_INTERRUPTED);
@@ -797,7 +917,7 @@ PUBLIC int HTFileCopy ARGS2(
     return rv;
 }
 
-#ifdef SOURCE_CACHE
+#ifdef USE_SOURCE_CACHE
 /*	Push data from an HTChunk down a stream
 **	---------------------------------------
 **
@@ -818,11 +938,12 @@ PUBLIC int HTMemCopy ARGS2(
 	HTChunk *,		chunk,
 	HTStream *,		sink)
 {
-    HTStreamClass targetClass = *(sink->isa);
+    HTStreamClass targetClass;
     int bytes = 0;
     CONST char *data = chunk->data;
     int rv = HT_OK;
 
+    targetClass = *(sink->isa);
     HTReadProgress(0, 0);
     for (;;) {
 	/* Push the data down the stream a piece at a time, in case we're
@@ -899,10 +1020,10 @@ PRIVATE int HTGzFileCopy ARGS2(
 		rv = HT_LOADED;
 		break;
 	    }
-	    CTRACE(tfp, "HTGzFileCopy: Read error, gzread returns %d\n",
-			status);
-	    CTRACE(tfp, "gzerror   : %s\n",
-			gzerror(gzfp, &gzerrnum));
+	    CTRACE((tfp, "HTGzFileCopy: Read error, gzread returns %d\n",
+			status));
+	    CTRACE((tfp, "gzerror   : %s\n",
+			gzerror(gzfp, &gzerrnum)));
 	    if (TRACE) {
 		if (gzerrnum == Z_ERRNO)
 		    perror("gzerror   ");
@@ -936,6 +1057,83 @@ PRIVATE int HTGzFileCopy ARGS2(
 }
 #endif /* USE_ZLIB */
 
+#ifdef USE_BZLIB
+/*	Push data from a bzip file pointer down a stream
+**	-------------------------------------
+**
+**   This routine is responsible for creating and PRESENTING any
+**   graphic (or other) objects described by the file.
+**
+**
+**  State of file and target stream on entry:
+**		      BZFILE (bzfp) assumed open (should have bzipped content),
+**		      target (sink) assumed valid.
+**
+**  Return values:
+**	HT_INTERRUPTED  Interruption after some data read.
+**	HT_PARTIAL_CONTENT	Error after some data read.
+**	-1		Error before any data read.
+**	HT_LOADED	Normal end of file indication on reading.
+**
+**  State of file and target stream on return:
+**	always		bzfp still open, target stream still valid.
+*/
+PRIVATE int HTBzFileCopy ARGS2(
+	BZFILE *,		bzfp,
+	HTStream*,		sink)
+{
+    HTStreamClass targetClass;
+    int status, bytes;
+    int bzerrnum;
+    int rv = HT_OK;
+
+    /*	Push the data down the stream
+    */
+    targetClass = *(sink->isa); /* Copy pointers to procedures */
+
+    /*	read and inflate bzip'd file, and push binary down sink
+    */
+    HTReadProgress(bytes = 0, 0);
+    for (;;) {
+	status = BZ2_bzread(bzfp, input_buffer, INPUT_BUFFER_SIZE);
+	if (status <= 0) { /* EOF or error */
+	    if (status == 0) {
+		rv = HT_LOADED;
+		break;
+	    }
+	    CTRACE((tfp, "HTBzFileCopy: Read error, bzread returns %d\n",
+			status));
+	    CTRACE((tfp, "bzerror   : %s\n",
+			BZ2_bzerror(bzfp, &bzerrnum)));
+	    if (bytes) {
+		rv = HT_PARTIAL_CONTENT;
+	    } else {
+		rv = -1;
+	    }
+	    break;
+	}
+
+	(*targetClass.put_block)(sink, input_buffer, status);
+	bytes += status;
+	HTReadProgress(bytes, -1);
+	HTDisplayPartial();
+
+	if (HTCheckForInterrupt()) {
+	    _HTProgress (TRANSFER_INTERRUPTED);
+	    if (bytes) {
+		rv = HT_INTERRUPTED;
+	    } else {
+		rv = -1;
+	    }
+	    break;
+	}
+    } /* next bufferload */
+
+    HTFinishDisplayPartial();
+    return rv;
+}
+#endif /* USE_BZLIB */
+
 /*	Push data from a socket down a stream STRIPPING CR
 **	--------------------------------------------------
 **
@@ -953,7 +1151,7 @@ PUBLIC void HTCopyNoCR ARGS3(
 	HTStream*,		sink)
 {
     HTStreamClass targetClass;
-    char character;
+    int character;
 
     /*	Push the data, ignoring CRLF, down the stream
     */
@@ -967,9 +1165,9 @@ PUBLIC void HTCopyNoCR ARGS3(
     HTInitInput(file_number);
     for (;;) {
 	character = HTGetCharacter();
-	if (character == (char)EOF)
+	if (character == EOF)
 	    break;
-	(*targetClass.put_character)(sink, character);
+	(*targetClass.put_character)(sink, UCH(character));
     }
 }
 
@@ -1028,7 +1226,7 @@ PUBLIC int HTParseSocket ARGS5(
 	}
 	HTSprintf0(&buffer, CANNOT_CONVERT_I_TO_O,
 		HTAtom_name(rep_in), HTAtom_name(format_out));
-	CTRACE(tfp, "HTFormat: %s\n", buffer);
+	CTRACE((tfp, "HTFormat: %s\n", buffer));
 	rv = HTLoadError(sink, 501, buffer); /* returns -501 */
 	FREE(buffer);
     } else {
@@ -1078,9 +1276,12 @@ PUBLIC int HTParseFile ARGS5(
     HTStreamClass targetClass;
     int rv;
 
-    stream = HTStreamStack(rep_in,
-			format_out,
-			sink , anchor);
+#ifdef SH_EX		/* 1998/01/04 (Sun) 16:04:09 */
+    if (fp == NULL)
+	return HT_LOADED;
+#endif
+
+    stream = HTStreamStack(rep_in, format_out, sink, anchor);
 
     if (!stream) {
 	char *buffer = 0;
@@ -1090,7 +1291,7 @@ PUBLIC int HTParseFile ARGS5(
 	}
 	HTSprintf0(&buffer, CANNOT_CONVERT_I_TO_O,
 		HTAtom_name(rep_in), HTAtom_name(format_out));
-	CTRACE(tfp, "HTFormat(in HTParseFile): %s\n", buffer);
+	CTRACE((tfp, "HTFormat(in HTParseFile): %s\n", buffer));
 	rv = HTLoadError(sink, 501, buffer);
 	FREE(buffer);
 	return rv;
@@ -1118,7 +1319,7 @@ PUBLIC int HTParseFile ARGS5(
 	return HT_LOADED;
 }
 
-#ifdef SOURCE_CACHE
+#ifdef USE_SOURCE_CACHE
 /*	Parse a document in memory given format and memory block pointer
 **
 **   This routine is responsible for creating and PRESENTING any
@@ -1132,7 +1333,7 @@ PUBLIC int HTParseFile ARGS5(
 **	-501		Stream stack failed (cannot present or convert).
 **	HT_LOADED	All data sent.
 **
-**  Stat of memory and target stream on return:
+**  State of memory and target stream on return:
 **	always		chunk unchanged; target freed, aborted, or NULL.
 */
 PUBLIC int HTParseMem ARGS5(
@@ -1151,7 +1352,7 @@ PUBLIC int HTParseMem ARGS5(
 	char *buffer = 0;
 	HTSprintf0(&buffer, CANNOT_CONVERT_I_TO_O,
 		   HTAtom_name(rep_in), HTAtom_name(format_out));
-	CTRACE(tfp, "HTFormat(in HTParseMem): %s\n", buffer);
+	CTRACE((tfp, "HTFormat(in HTParseMem): %s\n", buffer));
 	rv = HTLoadError(sink, 501, buffer);
 	FREE(buffer);
 	return rv;
@@ -1178,7 +1379,7 @@ PRIVATE int HTCloseGzFile ARGS1(
 	if (gzres == Z_ERRNO) {
 	    perror("gzclose   ");
 	} else if (gzres != Z_OK) {
-	    CTRACE(tfp, "gzclose   : error number %d\n", gzres);
+	    CTRACE((tfp, "gzclose   : error number %d\n", gzres));
 	}
     }
     return(gzres);
@@ -1211,9 +1412,7 @@ PUBLIC int HTParseGzFile ARGS5(
     HTStreamClass targetClass;
     int rv;
 
-    stream = HTStreamStack(rep_in,
-			format_out,
-			sink , anchor);
+    stream = HTStreamStack(rep_in, format_out, sink, anchor);
 
     if (!stream) {
 	char *buffer = 0;
@@ -1224,7 +1423,7 @@ PUBLIC int HTParseGzFile ARGS5(
 	}
 	HTSprintf0(&buffer, CANNOT_CONVERT_I_TO_O,
 		HTAtom_name(rep_in), HTAtom_name(format_out));
-	CTRACE(tfp, "HTFormat(in HTParseGzFile): %s\n", buffer);
+	CTRACE((tfp, "HTFormat(in HTParseGzFile): %s\n", buffer));
 	rv = HTLoadError(sink, 501, buffer);
 	FREE(buffer);
 	return rv;
@@ -1254,6 +1453,82 @@ PUBLIC int HTParseGzFile ARGS5(
 }
 #endif /* USE_ZLIB */
 
+#ifdef USE_BZLIB
+PRIVATE void HTCloseBzFile ARGS1(
+	BZFILE *,		bzfp)
+{
+    if (bzfp)
+	BZ2_bzclose(bzfp);
+}
+
+/*	HTParseBzFile
+**
+**  State of file and target stream on entry:
+**			bzFile (bzfp) assumed open,
+**			target (sink) usually NULL (will call stream stack).
+**
+**  Return values:
+**	-501		Stream stack failed (cannot present or convert).
+**	-1		Download cancelled.
+**	HT_NO_DATA	Error before any data read.
+**	HT_PARTIAL_CONTENT	Interruption or error after some data read.
+**	HT_LOADED	Normal end of file indication on reading.
+**
+**  State of file and target stream on return:
+**	always		bzfp closed; target freed, aborted, or NULL.
+*/
+PUBLIC int HTParseBzFile ARGS5(
+	HTFormat,		rep_in,
+	HTFormat,		format_out,
+	HTParentAnchor *,	anchor,
+	BZFILE*,		bzfp,
+	HTStream*,		sink)
+{
+    HTStream * stream;
+    HTStreamClass targetClass;
+    int rv;
+
+    stream = HTStreamStack(rep_in, format_out, sink, anchor);
+
+    if (!stream) {
+	char *buffer = 0;
+	HTCloseBzFile(bzfp);
+	if (LYCancelDownload) {
+	    LYCancelDownload = FALSE;
+	    return -1;
+	}
+	HTSprintf0(&buffer, CANNOT_CONVERT_I_TO_O,
+		HTAtom_name(rep_in), HTAtom_name(format_out));
+	CTRACE((tfp, "HTFormat(in HTParseBzFile): %s\n", buffer));
+	rv = HTLoadError(sink, 501, buffer);
+	FREE(buffer);
+	return rv;
+    }
+
+    /*	Push the data down the stream
+    **
+    **	@@  Bug:  This decision ought to be made based on "encoding"
+    **	rather than on content-type.  @@@  When we handle encoding.
+    **	The current method smells anyway.
+    */
+    targetClass = *(stream->isa);	/* Copy pointers to procedures */
+    rv = HTBzFileCopy(bzfp, stream);
+    if (rv == -1 || rv == HT_INTERRUPTED) {
+	(*targetClass._abort)(stream, NULL);
+    } else {
+	(*targetClass._free)(stream);
+    }
+
+    HTCloseBzFile(bzfp);
+    if (rv == -1)
+	return HT_NO_DATA;
+    else if (rv == HT_INTERRUPTED || (rv > 0 && rv != HT_LOADED))
+	return HT_PARTIAL_CONTENT;
+    else
+	return HT_LOADED;
+}
+#endif /* USE_BZLIB */
+
 /*	Converter stream: Network Telnet to internal character text
 **	-----------------------------------------------------------
 **
@@ -1276,7 +1551,7 @@ PRIVATE void NetToText_put_character ARGS2(HTStream *, me, char, net_char)
 	    me->sink->isa->put_character(me->sink, CR); /* leftover */
 	}
     }
-    me->had_cr = (c == CR);
+    me->had_cr = (BOOL) (c == CR);
     if (!me->had_cr)
 	me->sink->isa->put_character(me->sink, c);		/* normal */
 }
@@ -1324,7 +1599,7 @@ PRIVATE HTStreamClass NetToTextClass = {
 */
 PUBLIC HTStream * HTNetToText ARGS1(HTStream *, sink)
 {
-    HTStream* me = (HTStream*)malloc(sizeof(*me));
+    HTStream* me = typecalloc(HTStream);
 
     if (me == NULL)
 	outofmem(__FILE__, "NetToText");
@@ -1335,3 +1610,53 @@ PUBLIC HTStream * HTNetToText ARGS1(HTStream *, sink)
     return me;
 }
 
+PRIVATE HTStream	HTBaseStreamInstance;		      /* Made static */
+/*
+**	ERROR STREAM
+**	------------
+**	There is only one error stream shared by anyone who wants a
+**	generic error returned from all stream methods.
+*/
+PRIVATE void HTErrorStream_put_character ARGS2(HTStream *, me GCC_UNUSED, char, c GCC_UNUSED)
+{
+    LYCancelDownload = TRUE;
+}
+
+PRIVATE void HTErrorStream_put_string ARGS2(HTStream *, me GCC_UNUSED, CONST char *, s)
+{
+    if (s && *s)
+	LYCancelDownload = TRUE;
+}
+
+PRIVATE void HTErrorStream_write ARGS3(HTStream *, me GCC_UNUSED, CONST char *, s, int, l)
+{
+    if (l && s)
+	LYCancelDownload = TRUE;
+}
+
+PRIVATE void HTErrorStream_free ARGS1(HTStream *, me GCC_UNUSED)
+{
+    return;
+}
+
+PRIVATE void HTErrorStream_abort ARGS2(HTStream *, me GCC_UNUSED, HTError, e GCC_UNUSED)
+{
+    return;
+}
+
+PRIVATE CONST HTStreamClass HTErrorStreamClass =
+{
+    "ErrorStream",
+    HTErrorStream_free,
+    HTErrorStream_abort,
+    HTErrorStream_put_character,
+    HTErrorStream_put_string,
+    HTErrorStream_write
+};
+
+PUBLIC HTStream * HTErrorStream NOARGS
+{
+    CTRACE((tfp, "ErrorStream. Created\n"));
+    HTBaseStreamInstance.isa = &HTErrorStreamClass;    /* The rest is random */
+    return &HTBaseStreamInstance;
+}
