@@ -1,5 +1,6 @@
 #include <HTUtils.h>
 #include <HTAlert.h>
+#include <HTFile.h>
 #include <LYUtils.h>
 #include <LYStrings.h>
 #include <LYBookmark.h>
@@ -11,6 +12,7 @@
 #include <LYCharSets.h>  /* need for LYHaveCJKCharacterSet */
 #include <LYCurses.h>
 #include <GridText.h>
+#include <HTCJK.h>
 
 #ifdef VMS
 #include <nam.h>
@@ -24,6 +26,24 @@ PUBLIC char *MBM_A_subdescript[MBM_V_MAXFILES+1];
 PRIVATE BOOLEAN is_mosaic_hotlist = FALSE;
 PRIVATE char * convert_mosaic_bookmark_file PARAMS((char *filename_buffer));
 
+PUBLIC int LYindex2MBM ARGS1(int, n)
+{
+    static char MBMcodes[MBM_V_MAXFILES+2] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
+    return n >= 0 && n <= MBM_V_MAXFILES ? MBMcodes[n] : '?';
+}
+
+PUBLIC int LYMBM2index ARGS1(int, ch)
+{
+    if ((ch = TOUPPER(ch)) > 0) {
+	char *letters = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
+	char *result = strchr(letters, ch);
+	if (result != 0
+	 && (result - letters) <= MBM_V_MAXFILES)
+	    return (result - letters);
+    }
+    return -1;
+}
+
 PRIVATE void
 show_bookmark_not_defined NOARGS
 {
@@ -91,39 +111,33 @@ PUBLIC char * get_bookmark_filename ARGS1(
     LYAddPathToHome(filename_buffer,
 		    sizeof(filename_buffer),
 		    BookmarkPage);
-    CTRACE(tfp, "\nget_bookmark_filename: SEEKING %s\n   AS %s\n\n",
-		BookmarkPage, filename_buffer);
-    if ((fp = fopen(filename_buffer,"r")) != NULL) {
-	goto success;
-    }
-
-    /*
-     *	Failure.
-     */
-    return(NULL);
-
-success:
-    /*
-     *	We now have the file open.
-     *	Check if it is a mosaic hotlist.
-     */
-    if (LYSafeGets(&string_buffer, fp) != 0
-     && !strncmp(string_buffer, "ncsa-xmosaic-hotlist-format-1", 29)) {
-	char *newname;
+    CTRACE((tfp, "\nget_bookmark_filename: SEEKING %s\n   AS %s\n\n",
+		BookmarkPage, filename_buffer));
+    if ((fp = fopen(filename_buffer, TXT_R)) != NULL) {
 	/*
-	 *  It is a mosaic hotlist file.
+	 * We now have the file open.
+	 * Check if it is a mosaic hotlist.
 	 */
-	is_mosaic_hotlist = TRUE;
-	newname = convert_mosaic_bookmark_file(filename_buffer);
-	LYLocalFileToURL(URL, newname);
-    } else {
-	is_mosaic_hotlist = FALSE;
-	LYLocalFileToURL(URL, filename_buffer);
-    }
-    FREE(string_buffer);
-    fclose(fp);
+	if (LYSafeGets(&string_buffer, fp) != 0
+	 && *LYTrimNewline(string_buffer) != '\0'
+	 && !strncmp(string_buffer, "ncsa-xmosaic-hotlist-format-1", 29)) {
+	    char *newname;
+	    /*
+	     *  It is a mosaic hotlist file.
+	     */
+	    is_mosaic_hotlist = TRUE;
+	    newname = convert_mosaic_bookmark_file(filename_buffer);
+	    LYLocalFileToURL(URL, newname);
+	} else {
+	    is_mosaic_hotlist = FALSE;
+	    LYLocalFileToURL(URL, filename_buffer);
+	}
+	FREE(string_buffer);
+	LYCloseInput(fp);
 
-    return(filename_buffer);  /* bookmark file exists */
+	return(filename_buffer);  /* bookmark file exists */
+    }
+    return(NULL);
 
 } /* big end */
 
@@ -138,16 +152,15 @@ PRIVATE char * convert_mosaic_bookmark_file ARGS1(
     FILE *fp, *nfp;
     char *buf = NULL;
     int line = -2;
-    char *endline;
 
     LYRemoveTemp(newfile);
     if ((nfp = LYOpenTemp(newfile, HTML_SUFFIX, "w")) == NULL) {
 	LYMBM_statusline(NO_TEMP_FOR_HOTLIST);
-	sleep(AlertSecs);
+	LYSleepAlert();
 	return ("");
     }
 
-    if ((fp = fopen(filename_buffer, "r")) == NULL)
+    if ((fp = fopen(filename_buffer, TXT_R)) == NULL)
 	return ("");  /* should always open */
 
     fprintf(nfp,"<head>\n<title>%s</title>\n</head>\n",MOSAIC_BOOKMARK_TITLE);
@@ -158,11 +171,9 @@ PRIVATE char * convert_mosaic_bookmark_file ARGS1(
      been remapped by you or your system administrator."));
 
     while ((LYSafeGets(&buf, fp)) != NULL) {
-	if(line >= 0) {
-	    endline = &buf[strlen(buf)-1];
-	    if(*endline == '\n')
-		*endline = '\0';
-	    if((line % 2) == 0) { /* even lines */
+	if (line >= 0) {
+	    LYTrimNewline(buf);
+	    if ((line % 2) == 0) { /* even lines */
 		if(*buf != '\0') {
 		    strtok(buf," "); /* kill everything after the space */
 		    fprintf(nfp,"<LI><a href=\"%s\">",buf); /* the URL */
@@ -175,7 +186,7 @@ PRIVATE char * convert_mosaic_bookmark_file ARGS1(
 	line++;
     }
     LYCloseTempFP(nfp);
-    fclose(fp);
+    LYCloseInput(fp);
     return(newfile);
 }
 
@@ -200,6 +211,7 @@ PUBLIC void save_bookmark_link ARGS2(
     char *bookmark_URL = NULL;
     char filename_buffer[LY_MAXPATH];
     char string_buffer[BUFSIZ];
+    char tmp_buffer[BUFSIZ];
     char *Address = NULL;
     char *Title = NULL;
     int i, c;
@@ -237,7 +249,7 @@ PUBLIC void save_bookmark_link ARGS2(
 	    FREE(bookmark_URL);
 	    return;
 	}
-	strcpy(filename_buffer, filename);
+	LYstrncpy(filename_buffer, filename, sizeof(filename_buffer)-1);
     }
 
     /*
@@ -253,15 +265,18 @@ PUBLIC void save_bookmark_link ARGS2(
      *	If the link will be added to the same
      *	bookmark file, get confirmation. - FM
      */
-    if (LYMultiBookmarks == TRUE &&
-	strstr(HTLoadedDocumentURL(),
-	       (*BookmarkPage == '.' ?
-		    (BookmarkPage+1) : BookmarkPage)) != NULL) {
-	LYMBM_statusline(MULTIBOOKMARKS_SELF);
-	c = LYgetch();
-	if (TOUPPER(c) != 'L') {
-	    FREE(bookmark_URL);
-	    return;
+    if (LYMultiBookmarks != MBM_OFF) {
+	CONST char *url = HTLoadedDocumentURL();
+	CONST char *page = (*BookmarkPage == '.')
+			    ? (BookmarkPage + 1)
+			    : BookmarkPage;
+	if (strstr(url, page) != NULL) {
+	    LYMBM_statusline(MULTIBOOKMARKS_SELF);
+	    c = LYgetch_single();
+	    if (c != 'L') {
+		FREE(bookmark_URL);
+		return;
+	    }
 	}
     }
 
@@ -269,13 +284,27 @@ PUBLIC void save_bookmark_link ARGS2(
      *	Allow user to change the title. - FM
      */
     do {
-	LYstrncpy(string_buffer, title, sizeof(string_buffer)-1);
+	if (HTCJK == JAPANESE) {
+	    switch(kanji_code) {
+	    case EUC:
+		TO_EUC((CONST unsigned char *) title, (unsigned char *) tmp_buffer);
+		break;
+	    case SJIS:
+		TO_SJIS((CONST unsigned char *) title, (unsigned char *) tmp_buffer);
+		break;
+	    default:
+		break;
+	    }
+	    LYstrncpy(string_buffer, tmp_buffer, sizeof(string_buffer)-1);
+	} else {
+	    LYstrncpy(string_buffer, title, sizeof(string_buffer)-1);
+	}
 	convert_to_spaces(string_buffer, FALSE);
 	LYMBM_statusline(TITLE_PROMPT);
 	LYgetstr(string_buffer, VISIBLE, sizeof(string_buffer), NORECALL);
 	if (*string_buffer == '\0') {
 	    LYMBM_statusline(CANCELLED);
-	    sleep(MessageSecs);
+	    LYSleepMsg();
 	    FREE(bookmark_URL);
 	    return;
 	}
@@ -290,7 +319,7 @@ PUBLIC void save_bookmark_link ARGS2(
      *  from display character set which may need changing.
      *  Do NOT convert any 8-bit chars if we have CJK display. - LP
      */
-    StrAllocCopy(Title, string_buffer);
+    LYformTitle(&Title, string_buffer);
     LYEntify(&Title, TRUE);
     if (UCSaveBookmarksInUnicode &&
 	have8bit(Title) && (!LYHaveCJKCharacterSet)) {
@@ -312,11 +341,11 @@ PUBLIC void save_bookmark_link ARGS2(
 			sizeof(filename_buffer),
 			BookmarkPage);
     }
-    CTRACE(tfp, "\nsave_bookmark_link: SEEKING %s\n   AS %s\n\n",
-		BookmarkPage, filename_buffer);
-    if ((fp = fopen(filename_buffer, (first_time ? "w" : "a+"))) == NULL) {
+    CTRACE((tfp, "\nsave_bookmark_link: SEEKING %s\n   AS %s\n\n",
+		BookmarkPage, filename_buffer));
+    if ((fp = fopen(filename_buffer, (first_time ? TXT_W : TXT_A))) == NULL) {
 	LYMBM_statusline(BOOKMARK_OPEN_FAILED);
-	sleep(AlertSecs);
+	LYSleepAlert();
 	FREE(Title);
 	FREE(bookmark_URL);
 	return;
@@ -333,10 +362,23 @@ PUBLIC void save_bookmark_link ARGS2(
      *  Once and forever...
      */
     if (first_time) {
-	fprintf(fp,"<head>\n");
+	fprintf(fp, "<head>\n");
+#if defined(SH_EX) && !defined(_WINDOWS)	/* 1997/12/11 (Thu) 19:13:40 */
+	if (HTCJK != JAPANESE)
+	    LYAddMETAcharsetToFD(fp, -1);
+	else
+	    fprintf(fp, "<META %s %s>\n",
+		    "http-equiv=\"content-type\"",
+		    "content=\"text/html;charset=iso-2022-jp\"");
+#else
 	LYAddMETAcharsetToFD(fp, -1);
+#endif	/* !_WINDOWS */
 	fprintf(fp,"<title>%s</title>\n</head>\n", BOOKMARK_TITLE);
-	fprintf(fp, "%s<br>\n%s\n\n<!--\n%s\n-->\n\n<p>\n<ol>",
+#ifdef _WINDOWS
+	fprintf(fp,
+	    gettext("     You can delete links by the 'R' key<br>\n<ol>\n"));
+#else
+	fprintf(fp, "%s<br>\n%s\n\n<!--\n%s\n-->\n\n<p>\n<ol>\n",
 		    gettext("\
      You can delete links using the remove bookmark command.  It is usually\n\
      the 'R' key but may have been remapped by you or your system\n\
@@ -349,6 +391,7 @@ Note: if you edit this file manually\n\
       you should not change the format within the lines\n\
       or add other HTML markup.\n\
       Make sure any bookmark link is saved as a single line."));
+#endif	/* _WINDOWS */
     }
 
     /*
@@ -364,7 +407,7 @@ Note: if you edit this file manually\n\
     } else {
 	fprintf(fp,"<LI><a href=\"%s\">%s</a>\n", Address, Title);
     }
-    fclose(fp);
+    LYCloseOutput(fp);
 
     SetDefaultMode(O_BINARY);
     /*
@@ -374,18 +417,16 @@ Note: if you edit this file manually\n\
      */
     if (!first_time && nhist > 0 && bookmark_URL) {
 	for (i = 0; i < nhist; i++) {
-	    if (history[i].bookmark &&
-		!strcmp(history[i].address, bookmark_URL)) {
-		WWWDoc.address = history[i].address;
+	    if (HDOC(i).bookmark &&
+		!strcmp(HDOC(i).address, bookmark_URL)) {
+		WWWDoc.address = HDOC(i).address;
 		WWWDoc.post_data = NULL;
 		WWWDoc.post_content_type = NULL;
-		WWWDoc.bookmark = history[i].bookmark;
+		WWWDoc.bookmark = HDOC(i).bookmark;
 		WWWDoc.isHEAD = FALSE;
 		WWWDoc.safe = FALSE;
-		if (((tmpanchor = HTAnchor_parent(
-					HTAnchor_findAddress(&WWWDoc)
-						 )) != NULL) &&
-		    (text = (HText *)HTAnchor_document(tmpanchor)) != NULL) {
+		tmpanchor = HTAnchor_findAddress(&WWWDoc);
+		if ((text = (HText *)HTAnchor_document(tmpanchor)) != NULL) {
 		    HText_setNoCache(text);
 		}
 		break;
@@ -400,7 +441,7 @@ Note: if you edit this file manually\n\
     FREE(Address);
     FREE(bookmark_URL);
     LYMBM_statusline(OPERATION_DONE);
-    sleep(MessageSecs);
+    LYSleepMsg();
 }
 
 /*
@@ -420,31 +461,36 @@ PUBLIC void remove_bookmark_link ARGS2(
 #ifdef VMS
     char filename_buffer[NAM$C_MAXRSS+12];
     char newfile[NAM$C_MAXRSS+12];
+#define keep_tempfile FALSE
 #else
     char filename_buffer[LY_MAXPATH];
     char newfile[LY_MAXPATH];
+    BOOLEAN keep_tempfile = FALSE;
+#ifdef UNIX
     struct stat stat_buf;
     mode_t mode;
+    BOOLEAN regular = FALSE;
+#endif /* UNIX */
 #endif /* VMS */
     char homepath[LY_MAXPATH];
 
-    CTRACE(tfp, "remove_bookmark_link: deleting link number: %d\n", cur);
+    CTRACE((tfp, "remove_bookmark_link: deleting link number: %d\n", cur));
 
     if (!cur_bookmark_page)
 	return;
     LYAddPathToHome(filename_buffer,
 		    sizeof(filename_buffer),
 		    cur_bookmark_page);
-    CTRACE(tfp, "\nremove_bookmark_link: SEEKING %s\n   AS %s\n\n",
-		cur_bookmark_page, filename_buffer);
-    if ((fp = fopen(filename_buffer, "r")) == NULL) {
+    CTRACE((tfp, "\nremove_bookmark_link: SEEKING %s\n   AS %s\n\n",
+		cur_bookmark_page, filename_buffer));
+    if ((fp = fopen(filename_buffer, TXT_R)) == NULL) {
 	HTAlert(BOOKMARK_OPEN_FAILED_FOR_DEL);
 	return;
     }
 
     LYAddPathToHome(homepath, sizeof(homepath), "");
     if ((nfp = LYOpenScratch(newfile, homepath)) == 0) {
-	fclose(fp);
+	LYCloseInput(fp);
 	HTAlert(BOOKSCRA_OPEN_FAILED_FOR_DEL);
 	return;
     }
@@ -454,10 +500,11 @@ PUBLIC void remove_bookmark_link ARGS2(
      *	Explicitly preserve bookmark file mode on Unix. - DSL
      */
     if (stat(filename_buffer, &stat_buf) == 0) {
+	regular = (S_ISREG(stat_buf.st_mode) && stat_buf.st_nlink == 1);
 	mode = ((stat_buf.st_mode & 0777) | 0600); /* make it writable */
 	(void) chmod(newfile, mode);
 	if ((nfp = LYReopenTemp(newfile)) == NULL) {
-	    (void) fclose(fp);
+	    (void) LYCloseInput(fp);
 	    HTAlert(BOOKTEMP_REOPEN_FAIL_FOR_DEL);
 	    return;
 	}
@@ -476,15 +523,19 @@ PUBLIC void remove_bookmark_link ARGS2(
 	}
 
     } else {
-	char *cp;
+	char *cp, *cp2;
 	BOOLEAN retain;
 	int seen;
 
 	n = -1;
 	while (LYSafeGets(&buf, fp) != NULL) {
+	    int keep_ol = FALSE;
 	    retain = TRUE;
 	    seen = 0;
 	    cp = buf;
+	    if ((cur == 0) && (cp2 = LYstrstr(cp,"<ol><LI>")))
+		keep_ol = TRUE; /* Do not erase, this corrects a bug in an
+				   older version */
 	    while (n < cur && (cp = LYstrstr(cp, "<a href="))) {
 		seen++;
 		if (++n == cur) {
@@ -493,7 +544,9 @@ PUBLIC void remove_bookmark_link ARGS2(
 			HTAlert(BOOKMARK_LINK_NOT_ONE_LINE);
 			goto failure;
 		    }
-		    CTRACE(tfp, "remove_bookmark_link: skipping link %d\n", n);
+		    CTRACE((tfp, "remove_bookmark_link: skipping link %d\n", n));
+		    if (keep_ol)
+			fprintf(nfp,"<ol>\n");
 		    retain = FALSE;
 		}
 		cp += 8;
@@ -503,14 +556,19 @@ PUBLIC void remove_bookmark_link ARGS2(
 	}
     }
 
-    CTRACE(tfp, "remove_bookmark_link: files: %s %s\n",
-			newfile, filename_buffer);
+    FREE(buf);
+    CTRACE((tfp, "remove_bookmark_link: files: %s %s\n",
+			newfile, filename_buffer));
 
-    fclose(fp);
+    LYCloseInput(fp);
     fp = NULL;
+    if (fflush(nfp) == EOF) {
+	CTRACE((tfp, "fflush(nfp): %s", LYStrerror(errno)));
+	goto failure;
+    }
     LYCloseTempFP(nfp);
     nfp = NULL;
-#ifdef DOSPATH
+#if defined(DOSPATH) || defined(__EMX__)
     remove(filename_buffer);
 #endif /* DOSPATH */
 
@@ -520,12 +578,39 @@ PUBLIC void remove_bookmark_link ARGS2(
      *	can preserve the original ownership of the file, provided that
      *	it is writable by the current process.
      *	Changed to copy  1998-04-26 -- gil
+     *  But if the copy fails, for example because the filesystem is full,
+     *  we are left with a corrupt bookmark file.  Changed back to use
+     *  the previous mechanism [try rename(), then mv for EXDEV], except
+     *  in usual cases (not a regular file e.g., symbolic link, or has hard
+     *  links).  This will let bookmarks survive a filesystem full condition
+     *  in the "normal" case (bookmark is on same filesystem as home directory,
+     *  is a regular file, has no additional hard links).
+     *  If we first tried LYCopyFile, and that fails, also fall back to trying
+     *  the other stuff.  That gives a chance to recover in case the LYCopyFile
+     *  left a corrupt target file.
+     *  If there is an error, and that error may mean that the bookmark file
+     *  has been corrupted, don't remove the temporary newfile (which should
+     *  always be uncorrupted) in place, it may still be used to recover
+     *  manually.  If this applies, produce an additional message to that
+     *  effect.  The temp file will still be removed by normal program exit
+     *  cleanup. - kw 1999-11-12
      */
-    if (LYCopyFile(newfile, filename_buffer) == 0)
-	return;
-    HTAlert(BOOKTEMP_COPY_FAIL);
-#else  /* !UNIX */
+    if (!regular) {
+	if (LYCopyFile(newfile, filename_buffer) == 0) {
+	    LYRemoveTemp(newfile);
+	    return;
+	}
+	LYSleepAlert();	/* give a chance to see error from cp - kw */
+	HTUserMsg(BOOKTEMP_COPY_FAIL);
+	keep_tempfile = TRUE;
+    }
+#endif  /* UNIX */
+
     if (rename(newfile, filename_buffer) != -1) {
+#ifdef MULTI_USER_UNIX
+	if (regular)
+	    chmod(filename_buffer, stat_buf.st_mode & 07777);
+#endif
 	HTSYS_purge(filename_buffer);
 	return;
     } else {
@@ -535,22 +620,44 @@ PUBLIC void remove_bookmark_link ARGS2(
 	 *  Check if this is the case and do something appropriate.
 	 *  Used to be ODD_RENAME
 	 */
-#ifdef _WINDOWS
+#if defined(_WINDOWS) || defined(WIN_EX)
+#if defined(WIN_EX)
+	if (GetLastError() == ERROR_NOT_SAME_DEVICE)
+#else /* !_WIN_EX */
 	if (errno == ENOTSAM)
-#else
-	if (errno == EXDEV)
-#endif /* WINDOWS */
+#endif /* _WIN_EX */
 	{
+	    if (rename(newfile, filename_buffer) != 0) {
+		if (LYCopyFile(newfile, filename_buffer) == 0)
+		    remove(newfile);
+	    }
+	}
+#else
+	if (errno == EXDEV) {
 	    static CONST char MV_FMT[] = "%s %s %s";
 	    char *buffer = 0;
-	    HTAddParam(&buffer, MV_FMT, 1, MV_PATH);
-	    HTAddParam(&buffer, MV_FMT, 2, newfile);
-	    HTAddParam(&buffer, MV_FMT, 3, filename_buffer);
-	    HTEndParam(&buffer, MV_FMT, 3);
-	    LYSystem(buffer);
+	    CONST char *program;
+
+	    if ((program = HTGetProgramPath(ppMV)) != NULL) {
+		HTAddParam(&buffer, MV_FMT, 1, program);
+		HTAddParam(&buffer, MV_FMT, 2, newfile);
+		HTAddParam(&buffer, MV_FMT, 3, filename_buffer);
+		HTEndParam(&buffer, MV_FMT, 3);
+		if (LYSystem(buffer) == 0) {
+#ifdef MULTI_USER_UNIX
+		    if (regular)
+			chmod(filename_buffer, stat_buf.st_mode & 07777);
+#endif
+		    FREE(buffer);
+		    return;
+		}
+	    }
 	    FREE(buffer);
-	    return;
+	    keep_tempfile = TRUE;
+	    goto failure;
 	}
+	CTRACE((tfp, "rename(): %s", LYStrerror(errno)));
+#endif /* _WINDOWS */
 #endif /* !VMS */
 
 #ifdef VMS
@@ -561,15 +668,21 @@ PUBLIC void remove_bookmark_link ARGS2(
 	if (TRACE)
 	    perror("renaming the file");
     }
-#endif /* UNIX */
+
 
 failure:
     FREE(buf);
     HTAlert(BOOKMARK_DEL_FAILED);
-    LYCloseTempFP(nfp);
+    if (nfp)
+	LYCloseTempFP(nfp);
     if (fp != NULL)
-	fclose(fp);
-    LYRemoveTemp(newfile);
+	LYCloseInput(fp);
+    if (keep_tempfile) {
+	HTUserMsg2(gettext("File may be recoverable from %s during this session"),
+		   newfile);
+    } else {
+	LYRemoveTemp(newfile);
+    }
 }
 
 /*
@@ -582,7 +695,7 @@ PUBLIC int select_multi_bookmarks NOARGS
     /*
      *	If not enabled, pick the "default" (0).
      */
-    if (LYMultiBookmarks == FALSE || LYHaveSubBookmarks() == FALSE) {
+    if (LYMultiBookmarks == MBM_OFF || LYHaveSubBookmarks() == FALSE) {
 	if (MBM_A_subbookmark[0]) /* If it exists! */
 	    return(0);
 	else
@@ -594,18 +707,17 @@ PUBLIC int select_multi_bookmarks NOARGS
      *	the 2 redraws of the screen, if LYMBMAdvnced is TRUE.  '=' will
      *	still show the screen and let them do it the "long" way.
      */
-    if (LYMBMAdvanced && user_mode == ADVANCED_MODE) {
+    if (LYMultiBookmarks == MBM_ADVANCED && user_mode == ADVANCED_MODE) {
 	LYMBM_statusline(MULTIBOOKMARKS_SELECT);
 get_advanced_choice:
 	c = LYgetch();
 #ifdef VMS
 	if (HadVMSInterrupt) {
 	    HadVMSInterrupt = FALSE;
-	    c = 7;
+	    c = LYCharINTERRUPT2;
 	}
 #endif /* VMS */
-	if (LYisNonAlnumKeyname(c, LYK_PREV_DOC) ||
-	    c == 7 || c == 3) {
+	if (LYisNonAlnumKeyname(c, LYK_PREV_DOC) || LYCharIsINTERRUPT_HARD(c)) {
 	    /*
 	     *	Treat left-arrow, ^G, or ^C as cancel.
 	     */
@@ -616,7 +728,7 @@ get_advanced_choice:
 	     *	Refresh the screen.
 	     */
 	    lynx_force_repaint();
-	    refresh();
+	    LYrefresh();
 	    goto get_advanced_choice;
 	}
 	if (LYisNonAlnumKeyname(c, LYK_ACTIVATE)) {
@@ -637,8 +749,7 @@ get_advanced_choice:
 		 *  Convert to an array index, act on it if valid.
 		 *  Otherwise, get another keystroke.
 		 */
-		c = TOUPPER(c) - 'A';
-		if (c < 0 || c > MBM_V_MAXFILES) {
+		if ((c = LYMBM2index(c)) < 0) {
 		    goto get_advanced_choice;
 		}
 	}
@@ -659,20 +770,18 @@ get_advanced_choice:
  */
 PUBLIC int select_menu_multi_bookmarks NOARGS
 {
-    int c, MBM_tmp_count, MBM_allow;
+    int c, d, MBM_tmp_count, MBM_allow;
     int MBM_screens, MBM_from, MBM_to, MBM_current;
 
     /*
      *	If not enabled, pick the "default" (0).
      */
-    if (LYMultiBookmarks == FALSE)
+    if (LYMultiBookmarks == MBM_OFF)
 	return(0);
 
     /*
      *	Filip M. Gieszczykiewicz (filipg@paranoia.com) & FM
      *	---------------------------------------------------
-     *	LYMultiBookmarks - TRUE when multi_support enabled.
-     *
      *	MBM_A_subbookmark[n] - Hold values of the respective
      *	"multi_bookmarkn" in the lynxrc file.
      *
@@ -708,137 +817,128 @@ PUBLIC int select_menu_multi_bookmarks NOARGS
 
     MBM_current = 1; /* Gotta start somewhere :-) */
 
-draw_bookmark_choices:
-    MBM_from = MBM_allow * MBM_current - MBM_allow;
-    if (MBM_from < 0)
-	MBM_from = 0; /* 0 is default bookmark... */
-    if (MBM_current != 1)
-	MBM_from++;
+    for (;;) {
+	MBM_from = MBM_allow * MBM_current - MBM_allow;
+	if (MBM_from < 0)
+	    MBM_from = 0; /* 0 is default bookmark... */
+	if (MBM_current != 1)
+	    MBM_from++;
 
-    MBM_to = (MBM_allow * MBM_current);
-    if (MBM_to > MBM_V_MAXFILES)
-	MBM_to = MBM_V_MAXFILES;
+	MBM_to = (MBM_allow * MBM_current);
+	if (MBM_to > MBM_V_MAXFILES)
+	    MBM_to = MBM_V_MAXFILES;
 
-    /*
-     *	Display menu of bookmarks.  NOTE that we avoid printw()'s
-     *	to increase the chances that any non-ASCII or multibyte/CJK
-     *	characters will be handled properly. - FM
-     */
-    clear();
-    move(1, 5);
-    lynx_start_h1_color ();
-    if (MBM_screens > 1) {
-	char *shead_buffer = 0;
-	HTSprintf0(&shead_buffer,
-		MULTIBOOKMARKS_SHEAD_MASK, MBM_current, MBM_screens);
-	addstr(shead_buffer);
-	FREE(shead_buffer);
-    } else {
-	addstr(MULTIBOOKMARKS_SHEAD);
-    }
+	/*
+	 *  Display menu of bookmarks.  NOTE that we avoid printw()'s
+	 *  to increase the chances that any non-ASCII or multibyte/CJK
+	 *  characters will be handled properly. - FM
+	 */
+	LYclear();
+	LYmove(1, 5);
+	lynx_start_h1_color ();
+	if (MBM_screens > 1) {
+	    char *shead_buffer = 0;
+	    HTSprintf0(&shead_buffer,
+		    MULTIBOOKMARKS_SHEAD_MASK, MBM_current, MBM_screens);
+	    LYaddstr(shead_buffer);
+	    FREE(shead_buffer);
+	} else {
+	    LYaddstr(MULTIBOOKMARKS_SHEAD);
+	}
 
-    lynx_stop_h1_color ();
+	lynx_stop_h1_color ();
 
-    MBM_tmp_count = 0;
-    for (c = MBM_from; c <= MBM_to; c++) {
-	move(3+MBM_tmp_count, 5);
-	addch((unsigned char)(c + 'A'));
-	addstr(" : ");
-	if (MBM_A_subdescript[c])
-	    addstr(MBM_A_subdescript[c]);
-	move(3+MBM_tmp_count,36);
-	addch('(');
-	if (MBM_A_subbookmark[c])
-	    addstr(MBM_A_subbookmark[c]);
-	addch(')');
-	MBM_tmp_count++;
-    }
+	MBM_tmp_count = 0;
+	for (c = MBM_from; c <= MBM_to; c++) {
+	    LYmove(3+MBM_tmp_count, 5);
+	    LYaddch(LYindex2MBM(c));
+	    LYaddstr(" : ");
+	    if (MBM_A_subdescript[c])
+		LYaddstr(MBM_A_subdescript[c]);
+	    LYmove(3+MBM_tmp_count,36);
+	    LYaddch('(');
+	    if (MBM_A_subbookmark[c])
+		LYaddstr(MBM_A_subbookmark[c]);
+	    LYaddch(')');
+	    MBM_tmp_count++;
+	}
 
-    /*
-     *	Don't need to show it if it all fits on one screen!
-     */
-    if (MBM_screens > 1) {
-	move(LYlines-2, 0);
-	addstr("'");
-	standout();
-	addstr("[");
-	standend();
-	addstr("' ");
-	addstr(PREVIOUS);
-	addstr(", '");
-	standout();
-	addstr("]");
-	standend();
-	addstr("' ");
-	addstr(NEXT_SCREEN);
-    }
+	/*
+	 *  Don't need to show it if it all fits on one screen!
+	 */
+	if (MBM_screens > 1) {
+	    LYmove(LYlines-2, 0);
+	    LYaddstr("'");
+	    lynx_start_bold();
+	    LYaddstr("[");
+	    lynx_stop_bold();
+	    LYaddstr("' ");
+	    LYaddstr(PREVIOUS);
+	    LYaddstr(", '");
+	    lynx_start_bold();
+	    LYaddstr("]");
+	    lynx_stop_bold();
+	    LYaddstr("' ");
+	    LYaddstr(NEXT_SCREEN);
+	}
 
-    LYMBM_statusline(MULTIBOOKMARKS_SAVE);
-get_bookmark_choice:
-    c = LYgetch();
+	LYMBM_statusline(MULTIBOOKMARKS_SAVE);
+
+	for (;;) {
+	    c = LYgetch();
 #ifdef VMS
-    if (HadVMSInterrupt) {
-	HadVMSInterrupt = FALSE;
-	c = 7;
-    }
+	    if (HadVMSInterrupt) {
+		HadVMSInterrupt = FALSE;
+		c = 7;
+	    }
 #endif /* VMS */
 
-    if (LYisNonAlnumKeyname(c, LYK_PREV_DOC) ||
-	c == 7 || c == 3) {
-	/*
-	 *  Treat left-arrow, ^G, or ^C as cancel.
-	 */
-	return(-2);
-    }
+	    if ((d = LYMBM2index(c)) >= 0) {
+		/*
+		 *  See if we have a bookmark like that.
+		 */
+		if (MBM_A_subbookmark[d] != NULL)
+		    return(d);
 
-    if (LYisNonAlnumKeyname(c, LYK_REFRESH)) {
-	/*
-	 *  Refresh the screen.
-	 */
-	lynx_force_repaint();
-	refresh();
-	goto get_bookmark_choice;
-    }
+		show_bookmark_not_defined();
+		LYMBM_statusline(MULTIBOOKMARKS_SAVE);
+	    } else if (LYisNonAlnumKeyname(c, LYK_PREV_DOC) ||
+		c == 7 || c == 3) {
+		/*
+		 *  Treat left-arrow, ^G, or ^C as cancel.
+		 */
+		return(-2);
+	    } else if (LYisNonAlnumKeyname(c, LYK_REFRESH)) {
+		/*
+		 *  Refresh the screen.
+		 */
+		lynx_force_repaint();
+		LYrefresh();
+	    } else if (LYisNonAlnumKeyname(c, LYK_ACTIVATE)) {
+		/*
+		 *  Assume default bookmark file on ENTER or right-arrow.
+		 */
+		return(MBM_A_subbookmark[0] ? 0 : -1);
+	    } else if ((c == ']' ||  LYisNonAlnumKeyname(c, LYK_NEXT_PAGE)) &&
+		MBM_screens > 1) {
+		/*
+		 *  Next range, if available.
+		 */
+		if (++MBM_current > MBM_screens)
+		    MBM_current = 1;
+		break;
+	    }
 
-    if (LYisNonAlnumKeyname(c, LYK_ACTIVATE)) {
-	/*
-	 *  Assume default bookmark file on ENTER or right-arrow.
-	 */
-	return(MBM_A_subbookmark[0] ? 0 : -1);
-    }
-
-    /*
-     *	Next range, if available.
-     */
-    if ((c == ']' ||  LYisNonAlnumKeyname(c, LYK_NEXT_PAGE)) &&
-	MBM_screens > 1) {
-	if (++MBM_current > MBM_screens)
-	    MBM_current = 1;
-	goto draw_bookmark_choices;
-    }
-
-    /*
-     *	Previous range, if available.
-     */
-    if ((c == '[' ||  LYisNonAlnumKeyname(c, LYK_PREV_PAGE)) &&
-	MBM_screens > 1) {
-	if (--MBM_current <= 0)
-	    MBM_current = MBM_screens;
-	goto draw_bookmark_choices;
-    }
-
-    c = TOUPPER(c) - 'A';
-    /*
-     *	See if we have a bookmark like that.
-     */
-    if (c < 0 || c > MBM_V_MAXFILES) {
-	goto get_bookmark_choice;
-    } else if (!MBM_A_subbookmark[c]) {
-	show_bookmark_not_defined();
-	LYMBM_statusline(MULTIBOOKMARKS_SAVE);
-	goto get_bookmark_choice;
-    } else {
-	return(c);
+	    else if ((c == '[' ||  LYisNonAlnumKeyname(c, LYK_PREV_PAGE)) &&
+		MBM_screens > 1) {
+		/*
+		 *  Previous range, if available.
+		 */
+		if (--MBM_current <= 0)
+		    MBM_current = MBM_screens;
+		break;
+	    }
+	}
     }
 }
 
@@ -862,7 +962,7 @@ PUBLIC BOOLEAN LYHaveSubBookmarks NOARGS
 /*
  *  This function passes a string to _statusline(), making
  *  sure it is at the bottom of the screen if LYMultiBookmarks
- *  is TRUE, otherwise, letting it go to the normal statusline
+ *  is not MBM_OFF, otherwise, letting it go to the normal statusline
  *  position based on the current user mode.  We want to use
  *  _statusline() so that any multibyte/CJK characters in the
  *  string will be handled properly. - FM
@@ -870,7 +970,7 @@ PUBLIC BOOLEAN LYHaveSubBookmarks NOARGS
 PUBLIC void LYMBM_statusline  ARGS1(
 	char *,		text)
 {
-    if (LYMultiBookmarks == TRUE && user_mode == NOVICE_MODE) {
+    if (LYMultiBookmarks != MBM_OFF && user_mode == NOVICE_MODE) {
 	LYStatusLine = (LYlines - 1);
 	_statusline(text);
 	LYStatusLine = -1;
@@ -889,7 +989,7 @@ PRIVATE  BOOLEAN havevisible ARGS1(CONST char *, Title)
     long unicode;
 
     for ( ; *p; p++) {
-	c = (unsigned char)(TOASCII(*p));
+	c = UCH(TOASCII(*p));
 	if (c > 32 && c < 127)
 	    return(TRUE);
 	if (c <= 32 || c == 127)
@@ -916,7 +1016,7 @@ PRIVATE  BOOLEAN have8bit ARGS1(CONST char *, Title)
     CONST char *p = Title;
 
     for ( ; *p; p++) {
-	if ((unsigned char)*p > 127)
+	if (UCH(*p) > 127)
 	return(TRUE);
     }
     return(FALSE); /* if we came here */
@@ -955,7 +1055,7 @@ PRIVATE  char* title_convert8bit ARGS1(CONST char *, Title)
     for ( ; *p; p++) {
 	char temp[2];
 	LYstrncpy(temp, p, sizeof(temp)-1);
-	if ((unsigned char)*temp <= 127) {
+	if (UCH(*temp) <= 127) {
 	    StrAllocCat(comment, temp);
 	    StrAllocCat(ncr, temp);
 	} else {
@@ -980,7 +1080,7 @@ PRIVATE  char* title_convert8bit ARGS1(CONST char *, Title)
      *  skip '>'.
      */
     for (q = p0 = comment; *p0; p0++) {
-	if ((unsigned char)(TOASCII(*p0)) >= 32 &&
+	if (UCH(TOASCII(*p0)) >= 32 &&
 	    *p0 != '>' &&
 	    (q == comment || *p0 != '-' || *(q-1) != '-')) {
 	    *q++ = *p0;
@@ -1000,3 +1100,21 @@ PRIVATE  char* title_convert8bit ARGS1(CONST char *, Title)
     FREE(ncr);
     return(buf);
 }
+
+/*
+ * Since this is the "Default Bookmark File", we save it as a global, and as
+ * the first MBM_A_subbookmark entry.
+ */
+PUBLIC void set_default_bookmark_page ARGS1(
+	char *,		value)
+{
+    if (value != 0) {
+	if (bookmark_page == 0
+	 || strcmp(bookmark_page, value)) {
+	    StrAllocCopy(bookmark_page, value);
+	}
+	StrAllocCopy(BookmarkPage, bookmark_page);
+	StrAllocCopy(MBM_A_subbookmark[0], bookmark_page);
+	StrAllocCopy(MBM_A_subdescript[0], MULTIBOOKMARKS_DEFAULT);
+    }
+}
