@@ -4,32 +4,38 @@
 
 #include <LYGlobalDefs.h>
 #include <UCMap.h>
+#include <UCdomap.h>
 #include <UCDefs.h>
 #include <LYCharSets.h>
-#include <HTFont.h>
 #include <GridText.h>
 #include <LYCurses.h>
 #include <LYStrings.h>
 
 #include <LYLeaks.h>
 
-extern BOOL HTPassEightBitRaw;
-extern BOOL HTPassEightBitNum;
-extern BOOL HTPassHighCtrlRaw;
-extern BOOL HTPassHighCtrlNum;
-extern HTCJKlang HTCJK;
 PUBLIC HTkcode kanji_code = NOKANJI;
 PUBLIC BOOLEAN LYHaveCJKCharacterSet = FALSE;
 PUBLIC BOOLEAN DisplayCharsetMatchLocale = TRUE;
-PUBLIC BOOL force_old_UCLYhndl_on_reload;
+PUBLIC BOOL force_old_UCLYhndl_on_reload = FALSE;
 PUBLIC int forced_UCLYhdnl;
-extern void UCInit NOARGS;
-extern int UCInitialized;
 PUBLIC int LYNumCharsets = 0;  /* Will be initialized later by UC_Register. */
 PUBLIC int current_char_set = -1; /* will be intitialized later in LYMain.c */
 PUBLIC CONST char** p_entity_values = NULL; /* Pointer, for HTML_put_entity()*/
 			      /* obsolete and probably not used(???)        */
 			      /* will be initialized in HTMLUseCharacterSet */
+#ifdef EXP_CHARSET_CHOICE
+PUBLIC charset_subset_t charset_subsets[MAXCHARSETS];
+PUBLIC BOOL custom_display_charset = FALSE;
+PUBLIC BOOL custom_assumed_doc_charset = FALSE;
+#ifndef ALL_CHARSETS_IN_O_MENU_SCREEN
+PUBLIC int display_charset_map[MAXCHARSETS];
+PUBLIC int assumed_doc_charset_map[MAXCHARSETS];
+
+PUBLIC CONST char* display_charset_choices[MAXCHARSETS+1];
+PUBLIC CONST char* assumed_charset_choices[MAXCHARSETS+1];
+PUBLIC int displayed_display_charset_idx;
+#endif
+#endif /* EXP_CHARSET_CHOICE */
 
 /*
  *  New character sets now declared with UCInit() in UCdomap.c
@@ -372,8 +378,6 @@ PUBLIC LYUCcharset LYCharSet_UC[MAXCHARSETS]=
                        UCT_REP_SUBSETOF_LAT1,
                        UCT_CP_SUBSETOF_LAT1,   UCT_R_ASCII,UCT_R_ASCII},
 
-/*  {-1,"iso-8859-15",   UCT_ENC_8BIT,0,0,0,     UCT_R_8BIT,UCT_R_ASCII}, */
-
 };
 
 /*
@@ -406,19 +410,19 @@ PUBLIC void HTMLSetCharacterHandling ARGS1(int,i)
 	if (i == chndl)
 	    LYRawMode = LYUseDefaultRawMode;
 	else
-	    LYRawMode = (!LYUseDefaultRawMode);
+	    LYRawMode = (BOOL) (!LYUseDefaultRawMode);
 
-	HTPassEightBitNum =
+	HTPassEightBitNum = (BOOL) (
 	    ((LYCharSet_UC[i].codepoints & UCT_CP_SUPERSETOF_LAT1) ||
-		(LYCharSet_UC[i].like8859 & UCT_R_HIGH8BIT));
+		(LYCharSet_UC[i].like8859 & UCT_R_HIGH8BIT)));
 
 	if (LYRawMode) {
-	    HTPassEightBitRaw = (LYlowest_eightbit[i] <= 160);
+	    HTPassEightBitRaw = (BOOL) (LYlowest_eightbit[i] <= 160);
 	} else {
 	    HTPassEightBitRaw = FALSE;
 	}
 	if (LYRawMode || i == chndl) {
-	    HTPassHighCtrlRaw = (LYlowest_eightbit[i] <= 130);
+	    HTPassHighCtrlRaw = (BOOL) (LYlowest_eightbit[i] <= 130);
 	} else {
 	    HTPassHighCtrlRaw = FALSE;
 	}
@@ -448,10 +452,10 @@ PUBLIC void HTMLSetCharacterHandling ARGS1(int,i)
 	/* for any CJK: */
 	if (!LYUseDefaultRawMode)
 		HTCJK = NOCJK;
-	LYRawMode = (HTCJK != NOCJK) ? TRUE : FALSE;
+	LYRawMode = (BOOL) ((HTCJK != NOCJK) ? TRUE : FALSE);
 	HTPassEightBitRaw = FALSE;
 	HTPassEightBitNum = FALSE;
-	HTPassHighCtrlRaw = (HTCJK != NOCJK) ? TRUE : FALSE;
+	HTPassHighCtrlRaw = (BOOL) ((HTCJK != NOCJK) ? TRUE : FALSE);
 	HTPassHighCtrlNum = FALSE;
     }
 
@@ -484,18 +488,18 @@ PUBLIC void HTMLSetCharacterHandling ARGS1(int,i)
     }
 #endif /* USE_SLANG */
 
-    ena_csi((LYlowest_eightbit[current_char_set] > 155));
+    ena_csi((BOOLEAN)(LYlowest_eightbit[current_char_set] > 155));
 
     /* some diagnostics */
     if (TRACE) {
 	if (LYRawMode_flag != LYRawMode)
-	    CTRACE(tfp, "HTMLSetCharacterHandling: LYRawMode changed %s -> %s\n",
+	    CTRACE((tfp, "HTMLSetCharacterHandling: LYRawMode changed %s -> %s\n",
 			(LYRawMode_flag ? "ON" : "OFF"),
-			(LYRawMode	? "ON" : "OFF"));
+			(LYRawMode	? "ON" : "OFF")));
 	if (UCLYhndl_for_unspec_flag != UCLYhndl_for_unspec)
-	    CTRACE(tfp, "HTMLSetCharacterHandling: UCLYhndl_for_unspec changed %d -> %d\n",
+	    CTRACE((tfp, "HTMLSetCharacterHandling: UCLYhndl_for_unspec changed %d -> %d\n",
 			UCLYhndl_for_unspec_flag,
-			UCLYhndl_for_unspec);
+			UCLYhndl_for_unspec));
     }
 
     return;
@@ -542,7 +546,7 @@ PUBLIC void Set_HTCJK ARGS2(
  */
 PRIVATE void HTMLSetRawModeDefault ARGS1(int,i)
 {
-    LYDefaultRawMode = (LYCharSet_UC[i].enc == UCT_ENC_CJK);
+    LYDefaultRawMode = (BOOL) (LYCharSet_UC[i].enc == UCT_ENC_CJK);
     return;
 }
 
@@ -561,7 +565,7 @@ PUBLIC void HTMLSetUseDefaultRawMode ARGS2(
 	if (i == chndl)
 	    LYUseDefaultRawMode = modeflag;
 	else
-	    LYUseDefaultRawMode = (!modeflag);
+	    LYUseDefaultRawMode = (BOOL) (!modeflag);
     } else /* CJK encoding: */
 	    LYUseDefaultRawMode = modeflag;
 
@@ -574,7 +578,7 @@ PUBLIC void HTMLSetUseDefaultRawMode ARGS2(
  */
 PRIVATE void HTMLSetHaveCJKCharacterSet ARGS1(int,i)
 {
-    LYHaveCJKCharacterSet = (LYCharSet_UC[i].enc == UCT_ENC_CJK);
+    LYHaveCJKCharacterSet = (BOOL) (LYCharSet_UC[i].enc == UCT_ENC_CJK);
     return;
 }
 
@@ -608,7 +612,12 @@ PRIVATE void HTMLSetDisplayCharsetMatchLocale ARGS1(int,i)
 	match = TRUE; /* guess, but see below */
 
 #if !defined(LOCALE)
-	match = FALSE;
+	if (LYCharSet_UC[i].enc != UCT_ENC_UTF8)
+	    /*
+	     *  Leave true for utf-8 display - the code doesn't deal
+	     *  very well with this case. - kw
+	     */
+	    match = FALSE;
 #else
 	if (UCForce8bitTOUPPER) {
 	    /*
@@ -676,6 +685,8 @@ PRIVATE CONST names_pairs OLD_charset_names[] = {
     {"ISO 8859-6 Arabic",   "iso-8859-6"},
     {"ISO 8859-7 Greek",    "iso-8859-7"},
     {"ISO 8859-8 Hebrew",   "iso-8859-8"},
+    {"ISO-8859-8-I",        "iso-8859-8"},
+    {"ISO-8859-8-E",        "iso-8859-8"},
     {"ISO 8859-9 (Latin 5)","iso-8859-9"},
     {"ISO 8859-10",         "iso-8859-10"},
     {"UNICODE UTF 8",       "utf-8"},
@@ -697,6 +708,8 @@ PUBLIC int UCGetLYhndl_byAnyName ARGS1 (char *, value)
     LYTrimTrailing(value);
     if (value == NULL) return -1;
 
+    CTRACE((tfp, "UCGetLYhndl_byAnyName(%s)\n", value));
+
     /* search by name */
     for (i = 0; (i < MAXCHARSETS && LYchar_set_names[i]); i++) {
 	if (!strcmp(value, LYchar_set_names[i])) {
@@ -723,17 +736,17 @@ PUBLIC int UCGetLYhndl_byAnyName ARGS1 (char *, value)
  */
 PRIVATE CONST char * LYEntityNames[] = {
 /*	 NAME		   DECIMAL VALUE */
-	"nbsp", 	/* 160, non breaking space */
+	"nbsp",		/* 160, non breaking space */
 	"iexcl",	/* 161, inverted exclamation mark */
-	"cent", 	/* 162, cent sign */
+	"cent",		/* 162, cent sign */
 	"pound",	/* 163, pound sign */
 	"curren",	/* 164, currency sign */
 	"yen",		/* 165, yen sign */
 	"brvbar",	/* 166, broken vertical bar, (brkbar) */
-	"sect", 	/* 167, section sign */
+	"sect",		/* 167, section sign */
 	"uml",		/* 168, spacing dieresis */
-	"copy", 	/* 169, copyright sign */
-	"ordf", 	/* 170, feminine ordinal indicator */
+	"copy",		/* 169, copyright sign */
+	"ordf",		/* 170, feminine ordinal indicator */
 	"laquo",	/* 171, angle quotation mark, left */
 	"not",		/* 172, negation sign */
 	"shy",		/* 173, soft hyphen */
@@ -741,15 +754,15 @@ PRIVATE CONST char * LYEntityNames[] = {
 	"hibar",	/* 175, spacing macron */
 	"deg",		/* 176, degree sign */
 	"plusmn",	/* 177, plus-or-minus sign */
-	"sup2", 	/* 178, superscript 2 */
-	"sup3", 	/* 179, superscript 3 */
+	"sup2",		/* 178, superscript 2 */
+	"sup3",		/* 179, superscript 3 */
 	"acute",	/* 180, spacing acute (96) */
 	"micro",	/* 181, micro sign */
-	"para", 	/* 182, paragraph sign */
+	"para",		/* 182, paragraph sign */
 	"middot",	/* 183, middle dot */
 	"cedil",	/* 184, spacing cedilla */
-	"sup1", 	/* 185, superscript 1 */
-	"ordm", 	/* 186, masculine ordinal indicator */
+	"sup1",		/* 185, superscript 1 */
+	"ordm",		/* 186, masculine ordinal indicator */
 	"raquo",	/* 187, angle quotation mark, right */
 	"frac14",	/* 188, fraction 1/4 */
 	"frac12",	/* 189, fraction 1/2 */
@@ -759,31 +772,31 @@ PRIVATE CONST char * LYEntityNames[] = {
 	"Aacute",	/* 193, capital A, acute accent */
 	"Acirc",	/* 194, capital A, circumflex accent */
 	"Atilde",	/* 195, capital A, tilde */
-	"Auml", 	/* 196, capital A, dieresis or umlaut mark */
+	"Auml",		/* 196, capital A, dieresis or umlaut mark */
 	"Aring",	/* 197, capital A, ring */
 	"AElig",	/* 198, capital AE diphthong (ligature) */
 	"Ccedil",	/* 199, capital C, cedilla */
 	"Egrave",	/* 200, capital E, grave accent */
 	"Eacute",	/* 201, capital E, acute accent */
 	"Ecirc",	/* 202, capital E, circumflex accent */
-	"Euml", 	/* 203, capital E, dieresis or umlaut mark */
+	"Euml",		/* 203, capital E, dieresis or umlaut mark */
 	"Igrave",	/* 204, capital I, grave accent */
 	"Iacute",	/* 205, capital I, acute accent */
 	"Icirc",	/* 206, capital I, circumflex accent */
-	"Iuml", 	/* 207, capital I, dieresis or umlaut mark */
+	"Iuml",		/* 207, capital I, dieresis or umlaut mark */
 	"ETH",		/* 208, capital Eth, Icelandic (or Latin2 Dstrok) */
 	"Ntilde",	/* 209, capital N, tilde */
 	"Ograve",	/* 210, capital O, grave accent */
 	"Oacute",	/* 211, capital O, acute accent */
 	"Ocirc",	/* 212, capital O, circumflex accent */
 	"Otilde",	/* 213, capital O, tilde */
-	"Ouml", 	/* 214, capital O, dieresis or umlaut mark */
+	"Ouml",		/* 214, capital O, dieresis or umlaut mark */
 	"times",	/* 215, multiplication sign */
 	"Oslash",	/* 216, capital O, slash */
 	"Ugrave",	/* 217, capital U, grave accent */
 	"Uacute",	/* 218, capital U, acute accent */
 	"Ucirc",	/* 219, capital U, circumflex accent */
-	"Uuml", 	/* 220, capital U, dieresis or umlaut mark */
+	"Uuml",		/* 220, capital U, dieresis or umlaut mark */
 	"Yacute",	/* 221, capital Y, acute accent */
 	"THORN",	/* 222, capital THORN, Icelandic */
 	"szlig",	/* 223, small sharp s, German (sz ligature) */
@@ -791,34 +804,34 @@ PRIVATE CONST char * LYEntityNames[] = {
 	"aacute",	/* 225, small a, acute accent */
 	"acirc",	/* 226, small a, circumflex accent */
 	"atilde",	/* 227, small a, tilde */
-	"auml", 	/* 228, small a, dieresis or umlaut mark */
+	"auml",		/* 228, small a, dieresis or umlaut mark */
 	"aring",	/* 229, small a, ring */
 	"aelig",	/* 230, small ae diphthong (ligature) */
 	"ccedil",	/* 231, small c, cedilla */
 	"egrave",	/* 232, small e, grave accent */
 	"eacute",	/* 233, small e, acute accent */
 	"ecirc",	/* 234, small e, circumflex accent */
-	"euml", 	/* 235, small e, dieresis or umlaut mark */
+	"euml",		/* 235, small e, dieresis or umlaut mark */
 	"igrave",	/* 236, small i, grave accent */
 	"iacute",	/* 237, small i, acute accent */
 	"icirc",	/* 238, small i, circumflex accent */
-	"iuml", 	/* 239, small i, dieresis or umlaut mark */
+	"iuml",		/* 239, small i, dieresis or umlaut mark */
 	"eth",		/* 240, small eth, Icelandic */
 	"ntilde",	/* 241, small n, tilde */
 	"ograve",	/* 242, small o, grave accent */
 	"oacute",	/* 243, small o, acute accent */
 	"ocirc",	/* 244, small o, circumflex accent */
 	"otilde",	/* 245, small o, tilde */
-	"ouml", 	/* 246, small o, dieresis or umlaut mark */
+	"ouml",		/* 246, small o, dieresis or umlaut mark */
 	"divide",	/* 247, division sign */
 	"oslash",	/* 248, small o, slash */
 	"ugrave",	/* 249, small u, grave accent */
 	"uacute",	/* 250, small u, acute accent */
 	"ucirc",	/* 251, small u, circumflex accent */
-	"uuml", 	/* 252, small u, dieresis or umlaut mark */
+	"uuml",		/* 252, small u, dieresis or umlaut mark */
 	"yacute",	/* 253, small y, acute accent */
 	"thorn",	/* 254, small thorn, Icelandic */
-	"yuml", 	/* 255, small y, dieresis or umlaut mark */
+	"yuml",		/* 255, small y, dieresis or umlaut mark */
 };
 
 /*
@@ -829,7 +842,7 @@ PUBLIC CONST char * HTMLGetEntityName ARGS1(
 	UCode_t,	code)
 {
 #define IntValue code
-    int MaxValue = ((sizeof(LYEntityNames)/sizeof(char **)) - 1);
+    int MaxValue = (TABLESIZE(LYEntityNames) - 1);
 
     if (IntValue < 0 || IntValue > MaxValue) {
 	return "";
@@ -859,7 +872,7 @@ PUBLIC UCode_t HTMLGetEntityUCValue ARGS1(
     UCode_t value = 0;
     size_t i, high, low;
     int diff = 0;
-    size_t number_of_unicode_entities = sizeof(unicode_entities)/sizeof(unicode_entities[0]);
+    size_t number_of_unicode_entities = TABLESIZE(unicode_entities);
 
     /*
      *	Make sure we have a non-zero length name. - FM
@@ -904,9 +917,43 @@ PUBLIC void HTMLUseCharacterSet ARGS1(int, i)
  *  Initializer, calls initialization function for the
  *  CHARTRANS handling. - KW
  */
-PUBLIC int LYCharSetsDeclared NOPARAMS
+PUBLIC int LYCharSetsDeclared NOARGS
 {
     UCInit();
 
     return UCInitialized;
 }
+
+#ifdef EXP_CHARSET_CHOICE
+PUBLIC void init_charset_subsets NOARGS
+{
+    int i,n;
+    int cur_display = 0;
+    int cur_assumed = 0;
+
+    /* add them to displayed values */
+    charset_subsets[UCLYhndl_for_unspec].hide_assumed = FALSE;
+    charset_subsets[current_char_set].hide_display = FALSE;
+
+#ifndef ALL_CHARSETS_IN_O_MENU_SCREEN
+    /*all this stuff is for supporting old menu screen... */
+    for (i = 0; i < LYNumCharsets; ++i){
+	if (charset_subsets[i].hide_display == FALSE) {
+	    n = cur_display++;
+	    if (i == current_char_set)
+		displayed_display_charset_idx = n;
+	    display_charset_map[n] = i;
+	    display_charset_choices[n] = LYchar_set_names[i];
+	}
+	if (charset_subsets[i].hide_assumed == FALSE) {
+	    n = cur_assumed++;
+	    assumed_doc_charset_map[n] = i;
+	    assumed_charset_choices[n] = LYCharSet_UC[i].MIMEname;
+	    charset_subsets[i].assumed_idx = n;
+	}
+	display_charset_choices[cur_display] = NULL;
+	assumed_charset_choices[cur_assumed] = NULL;
+    }
+#endif
+}
+#endif /* EXP_CHARSET_CHOICE */
