@@ -5,10 +5,9 @@
 #include <LYStructs.h>
 #endif /* LYSTRUCTS_H */
 
-extern int printfile PARAMS((document *newdoc));
+extern int printfile PARAMS((DocInfo *newdoc));
 extern int print_options PARAMS((char **newfile,
-				 char **printed_url, int lines_in_file));
+				 CONST char *printed_url, int lines_in_file));
 extern char * GetFileName NOPARAMS;
 
 #endif /* LYPRINT_H */
-
